@@ -24,13 +24,17 @@ import java.util.Map;
 
 import net.sf.ehcache.bootstrap.BootstrapCacheLoader;
 import net.sf.ehcache.config.CacheConfiguration;
+import net.sf.ehcache.config.NonstopConfiguration;
 import net.sf.ehcache.event.RegisteredEventListeners;
 import net.sf.ehcache.exceptionhandler.CacheExceptionHandler;
 import net.sf.ehcache.extension.CacheExtension;
 import net.sf.ehcache.loader.CacheLoader;
+import net.sf.ehcache.search.Attribute;
+import net.sf.ehcache.search.Query;
 import net.sf.ehcache.statistics.CacheUsageListener;
 import net.sf.ehcache.statistics.LiveCacheStatistics;
 import net.sf.ehcache.statistics.sampled.SampledCacheStatistics;
+import net.sf.ehcache.terracotta.TerracottaNotRunningException;
 import net.sf.ehcache.transaction.manager.TransactionManagerLookup;
 import net.sf.ehcache.writer.CacheWriter;
 import net.sf.ehcache.writer.CacheWriterManager;
@@ -50,9 +54,31 @@ import net.sf.ehcache.writer.CacheWriterManager;
  * Statistics on cache usage are collected and made available through public methods.
  *
  * @author Greg Luck
- * @version $Id: Ehcache.java 2269 2010-04-16 21:30:10Z gkeim $
+ * @version $Id: Ehcache.java 4843 2011-10-15 09:04:35Z amaheshw $
  */
 public interface Ehcache extends Cloneable {
+
+    /**
+     * unpin all pinned keys
+     */
+    void unpinAll();
+
+    /**
+     * Check if the key is pinned
+     * @param key the key to be checked
+     * @return true if the element is pinned
+     */
+    boolean isPinned(Object key);
+
+    /**
+     * Mark the key as pinned or not. Pinning is associated with key. Once the key is pinned, the mapping is held in the memory
+     * until setPinned(key, false) is called. {@link #replace} and {@link #remove} does not change the pin state of key. This
+     * comes with little memory overhead that even if key is removed, it is still held in memory.
+     * @param key the key to be pinned or not
+     * @param pinned {@code true} if the key should be pinned, false otherwise
+     */
+    void setPinned(Object key, boolean pinned);
+
     /**
      * Put an element in the cache.
      * <p/>
@@ -74,6 +100,25 @@ public interface Ehcache extends Cloneable {
     void put(Element element) throws IllegalArgumentException, IllegalStateException,
             CacheException;
 
+
+    /**
+     * Puts a collection of elements in the cache. Throws a NullPointerException if any element in the
+     * collection is null. Also notifies the CacheEventListener that:
+     * <ul>
+     * <li>the elements were put. The puts{@link net.sf.ehcache.Ehcache#put(net.sf.ehcache.Element)} happen in batches and the notifications are thrown for every put in the
+     * batch irrespective of whether the element is present in the cache or not i.e this method consider
+     * each element as new entry.
+     * </li>
+     * </ul>
+     * This operation is partially completed if any element or any key is null
+     * @param elements a collection of elements to be put in the cache.
+     *        If elements are Serializable it can fully participate in replication and the DiskStore.
+     * @throws IllegalStateException    if the cache is not {@link net.sf.ehcache.Status#STATUS_ALIVE}
+     * @throws CacheException
+     */
+    void putAll(Collection<Element> elements) throws IllegalArgumentException, IllegalStateException,
+            CacheException;
+
     /**
      * Put an element in the cache.
      * <p/>
@@ -131,28 +176,28 @@ public interface Ehcache extends Cloneable {
 
     /**
      * Put an element in the cache if no element is currently mapped to the elements key.
-     * 
+     *
      * @param element element to be added
      * @return the element previously cached for this key, or null if none.
-     * 
+     *
      * @throws NullPointerException if the element is null, or has a null key
      */
     Element putIfAbsent(Element element) throws NullPointerException;
-    
+
     /**
      * Remove the Element mapped to the key for the supplied element if the value of the supplied Element
      * is equal to the value of the cached Element.
-     * 
+     *
      * @param element Element to be removed
      * @return true if the value was removed
-     * 
+     *
      * @throws NullPointerException if the element is null, or has a null key
      */
     boolean removeElement(Element element) throws NullPointerException;
 
     /**
      * Replace the cached element only if the current Element is equal to the supplied old Element.
-     * 
+     *
      * @param old Element to be test against
      * @param element Element to be cached
      * @return true is the Element was replaced
@@ -168,7 +213,7 @@ public interface Ehcache extends Cloneable {
      * @throws NullPointerException if the Element is null or has a null key
      */
     Element replace(Element element) throws NullPointerException;
-    
+
     /**
      * Gets an element from the cache. Updates Element Statistics
      * <p/>
@@ -186,7 +231,7 @@ public interface Ehcache extends Cloneable {
      * Gets an element from the cache. Updates Element Statistics
      * <p/>
      * Note that the Element's lastAccessTime is always the time of this get.
-     * Use {@link #getQuiet(Object)} to peak into the Element to see its last access time with get
+     * Use {@link #getQuiet(Object)} to peek into the Element to see its last access time with get
      *
      * @param key an Object value
      * @return the element, or null, if it does not exist.
@@ -196,6 +241,21 @@ public interface Ehcache extends Cloneable {
      */
     Element get(Object key) throws IllegalStateException, CacheException;
 
+    /**
+     * Gets all the elements from the cache for the keys provided. Updates Element Statistics
+     * Throws a NullPointerException if any key in the collection is null
+     * <p/>
+     * Note that the Element's lastAccessTime is always the time of this get.
+     * Use {@link #getQuiet(Object)} to peek into the Element to see its last access time with get
+     *
+     * @param keys a collection of keys for which value is to be fetched
+     * @return Map of key and elements for the provided keys, value will be null for the keys which do not exist
+     * @throws IllegalStateException if the cache is not {@link net.sf.ehcache.Status#STATUS_ALIVE}
+     * @throws NullPointerException if any key is null in the collection
+     * @see #isExpired
+     */
+    Map<Object, Element> getAll(Collection<?> keys) throws IllegalStateException, CacheException, NullPointerException;
+
     /**
      * Gets an element from the cache, without updating Element statistics. Cache statistics are
      * still updated.
@@ -205,6 +265,7 @@ public interface Ehcache extends Cloneable {
      * @return the element, or null, if it does not exist.
      * @throws IllegalStateException if the cache is not {@link net.sf.ehcache.Status#STATUS_ALIVE}
      * @see #isExpired
+     * @since 2.5
      */
     Element getQuiet(Serializable key) throws IllegalStateException, CacheException;
 
@@ -275,7 +336,9 @@ public interface Ehcache extends Cloneable {
      *
      * @return a list of {@link Object} keys
      * @throws IllegalStateException if the cache is not {@link net.sf.ehcache.Status#STATUS_ALIVE}
+     * @deprecated versions since 2.1 do not return duplicates
      */
+    @Deprecated
     List getKeysNoDuplicateCheck() throws IllegalStateException;
 
     /**
@@ -304,6 +367,31 @@ public interface Ehcache extends Cloneable {
      */
     boolean remove(Object key) throws IllegalStateException;
 
+    /**
+     * Removes given set of {@link net.sf.ehcache.Element} from the Cache. This also removes them from any
+     * stores it may be in. Throws a NullPointerException if any key in the collection is null
+     * <p/>
+     * Also notifies the CacheEventListener after the elements were removed.
+     * Notification is sent for every key irrespective of whether the key was present in the cache or not
+     * This operation is partially completed if any element or any key is null
+     * @param keys                   a collection of keys to operate on
+     * @throws IllegalStateException    if the cache is not {@link net.sf.ehcache.Status#STATUS_ALIVE}
+     * @throws NullPointerException     if any key is null in the collection
+     */
+    void removeAll(Collection<?> keys) throws IllegalStateException, NullPointerException;
+
+    /**
+     * Removes all cached items.
+     * <p />
+     * When using Terracotta clustered caches with nonstop enabled, the timeout used by this method is
+     * {@link NonstopConfiguration#getBulkOpsTimeoutMultiplyFactor()} times the timeout value in the nonstop config.
+     *
+     * @param doNotNotifyCacheReplicators whether the put is coming from a doNotNotifyCacheReplicators cache peer,
+     *                                    in which case this put should not initiate a further notification to doNotNotifyCacheReplicators cache peers
+     * @throws IllegalStateException if the cache is not {@link net.sf.ehcache.Status#STATUS_ALIVE}
+     */
+    void removeAll(Collection<?> keys, boolean doNotNotifyCacheReplicators) throws IllegalStateException, NullPointerException;
+
     /**
      * Removes an {@link net.sf.ehcache.Element} from the Cache. This also removes it from any
      * stores it may be in.
@@ -361,7 +449,7 @@ public interface Ehcache extends Cloneable {
      * Removes an {@link net.sf.ehcache.Element} from the Cache and any stores it might be in. This also removes through
      * to a CacheWriter. If no CacheWriter has been registered for the cache, then this method throws an exception.
      * <p/>
-     * Also notifies the CacheEventListener after the element was removed, but only if an El ement
+     * Also notifies the CacheEventListener after the element was removed, but only if an Element
      * with the key actually existed.
      *
      * @param key
@@ -374,6 +462,9 @@ public interface Ehcache extends Cloneable {
 
     /**
      * Removes all cached items.
+     * <p />
+     * When using Terracotta clustered caches with nonstop enabled, the timeout used by this method is
+     * {@link NonstopConfiguration#getBulkOpsTimeoutMultiplyFactor()} times the timeout value in the nonstop config.
      *
      * @throws IllegalStateException if the cache is not {@link net.sf.ehcache.Status#STATUS_ALIVE}
      */
@@ -381,6 +472,9 @@ public interface Ehcache extends Cloneable {
 
     /**
      * Removes all cached items.
+     * <p />
+     * When using Terracotta clustered caches with nonstop enabled, the timeout used by this method is
+     * {@link NonstopConfiguration#getBulkOpsTimeoutMultiplyFactor()} times the timeout value in the nonstop config.
      *
      * @param doNotNotifyCacheReplicators whether the put is coming from a doNotNotifyCacheReplicators cache peer,
      *                                    in which case this put should not initiate a further notification to doNotNotifyCacheReplicators cache peers
@@ -399,7 +493,7 @@ public interface Ehcache extends Cloneable {
      * Gets the size of the cache. This is a subtle concept. See below.
      * <p/>
      * The size is the number of {@link net.sf.ehcache.Element}s in the {@link net.sf.ehcache.store.MemoryStore} plus
-     * the number of {@link net.sf.ehcache.Element}s in the {@link net.sf.ehcache.store.DiskStore}.
+     * the number of {@link net.sf.ehcache.Element}s in the {@link net.sf.ehcache.store.disk.DiskStore}.
      * <p/>
      * This number is the actual number of elements, including expired elements that have
      * not been removed.
@@ -416,16 +510,18 @@ public interface Ehcache extends Cloneable {
      * <p/>
      * To get a very fast result, use {@link #getKeysNoDuplicateCheck()}.size(). If the disk store
      * is being used, there will be some duplicates.
+     * <p/>
+     * Note:getSize() is a very expensive operation in off-heap, disk and Terracotta implementations.
      *
      * @return The size value
      * @throws IllegalStateException if the cache is not {@link net.sf.ehcache.Status#STATUS_ALIVE}
      */
     int getSize() throws IllegalStateException, CacheException;
-    
+
     /**
      * Accurately measuring statistics can be expensive. Returns the size of the
      * cache based on the accuracy setting
-     * 
+     *
      * @param statisticsAccuracy
      *            one of {@link Statistics#STATISTICS_ACCURACY_BEST_EFFORT},
      *            {@link Statistics#STATISTICS_ACCURACY_GUARANTEED},
@@ -454,6 +550,29 @@ public interface Ehcache extends Cloneable {
      */
     long calculateInMemorySize() throws IllegalStateException, CacheException;
 
+    /**
+     * Gets the size of the off-heap store for this cache
+     *
+     * @return the size of the off-heap store in bytes
+     * @throws IllegalStateException
+     */
+    long calculateOffHeapSize() throws IllegalStateException, CacheException;
+
+    /**
+     * Gets the size of the on-disk store for this cache
+     *
+     * @return the size of the on-disk store in bytes
+     * @throws IllegalStateException
+     */
+    long calculateOnDiskSize() throws IllegalStateException, CacheException;
+
+    /**
+     * Check if the cache may contain elements which the SizeOf engine could not fully size.
+     *
+     * @return true if the cache may contain partially sized objects
+     */
+    boolean hasAbortedSizeOf();
+
     /**
      * Returns the number of elements in the memory store.
      *
@@ -462,6 +581,14 @@ public interface Ehcache extends Cloneable {
      */
     long getMemoryStoreSize() throws IllegalStateException;
 
+    /**
+     * Returns the number of elements in the off-heap store.
+     *
+     * @return the number of elements in the off-heap store
+     * @throws IllegalStateException if the cache is not {@link net.sf.ehcache.Status#STATUS_ALIVE}
+     */
+    long getOffHeapStoreSize() throws IllegalStateException;
+
     /**
      * Returns the number of elements in the disk store.
      *
@@ -514,7 +641,7 @@ public interface Ehcache extends Cloneable {
     /**
      * Clones a cache. This is only legal if the cache has not been
      * initialized. At that point only primitives have been set and no
-     * {@link net.sf.ehcache.store.MemoryStore} or {@link net.sf.ehcache.store.DiskStore} has been created.
+     * {@link net.sf.ehcache.store.MemoryStore} or {@link net.sf.ehcache.store.disk.DiskStore} has been created.
      * <p/>
      * A new, empty, RegisteredEventListeners is created on clone.
      * <p/>
@@ -534,6 +661,9 @@ public interface Ehcache extends Cloneable {
 
     /**
      * Whether an Element is stored in the cache in Memory, indicating a very low cost of retrieval.
+     * <p>
+     * Since no assertions are made about the state of the Element it is possible that the
+     * Element is expired, but this method still returns true.
      *
      * @return true if an element matching the key is found in memory
      */
@@ -541,6 +671,9 @@ public interface Ehcache extends Cloneable {
 
     /**
      * Whether an Element is stored in the cache in Memory, indicating a very low cost of retrieval.
+     * <p>
+     * Since no assertions are made about the state of the Element it is possible that the
+     * Element is expired, but this method still returns true.
      *
      * @return true if an element matching the key is found in memory
      * @since 1.2
@@ -549,6 +682,9 @@ public interface Ehcache extends Cloneable {
 
     /**
      * Whether an Element is stored in the cache on Disk, indicating a higher cost of retrieval.
+     * <p>
+     * Since no assertions are made about the state of the Element it is possible that the
+     * Element is expired, but this method still returns true.
      *
      * @return true if an element matching the key is found in the diskStore
      */
@@ -556,6 +692,9 @@ public interface Ehcache extends Cloneable {
 
     /**
      * Whether an Element is stored in the cache on Disk, indicating a higher cost of retrieval.
+     * <p>
+     * Since no assertions are made about the state of the Element it is possible that the
+     * Element is expired, but this method still returns true.
      *
      * @return true if an element matching the key is found in the diskStore
      * @since 1.2
@@ -607,6 +746,9 @@ public interface Ehcache extends Cloneable {
 
     /**
      * An inexpensive check to see if the key exists in the cache.
+     * <p>
+     * Since no assertions are made about the state of the Element it is possible that the
+     * Element is expired, but this method still returns true.
      *
      * @param key the key to check for
      * @return true if an Element matching the key is found in the cache. No assertions are made about the state of the Element.
@@ -629,7 +771,7 @@ public interface Ehcache extends Cloneable {
      * This result is returned when the statistics accuracy setting is {@link Statistics#STATISTICS_ACCURACY_BEST_EFFORT}.
      * <p/>
      * The size is the number of {@link Element}s in the {@link net.sf.ehcache.store.MemoryStore} plus
-     * the number of {@link Element}s in the {@link net.sf.ehcache.store.DiskStore}.
+     * the number of {@link Element}s in the {@link net.sf.ehcache.store.disk.DiskStore}.
      * <p/>
      * This number is the actual number of elements, including expired elements that have
      * not been removed. Any duplicates between stores are accounted for.
@@ -656,13 +798,13 @@ public interface Ehcache extends Cloneable {
      * @throws IllegalStateException if the cache is not {@link Status#STATUS_ALIVE}
      */
     Statistics getStatistics() throws IllegalStateException;
-    
+
     /**
      * This is different from {@link #getStatistics()} in the way that values
      * returned from {@link LiveCacheStatistics} will reflect the current state
      * of the cache (and not a snapshot of the cache when the api's were called
      * like {@link #getStatistics()})
-     * 
+     *
      * @return The {@link LiveCacheStatistics} associated with this cache
      * @throws IllegalStateException
      * @since 1.7
@@ -676,7 +818,7 @@ public interface Ehcache extends Cloneable {
      * Implementations of {@link CacheUsageListener} should override the
      * {@link Object#equals(Object)} and {@link Object#hashCode()} methods as it is used for
      * equality check
-     * 
+     *
      * @throws IllegalStateException
      * @since 1.7
      */
@@ -686,7 +828,7 @@ public interface Ehcache extends Cloneable {
     /**
      * Remove an already registered {@link CacheUsageListener}, if any.
      * Depends on the {@link Object#equals(Object)} method.
-     * 
+     *
      * @throws IllegalStateException
      * @since 1.7
      */
@@ -726,7 +868,7 @@ public interface Ehcache extends Cloneable {
     void setDiskStorePath(String diskStorePath) throws CacheException;
 
     /**
-     * Newly created caches do not have a {@link net.sf.ehcache.store.MemoryStore} or a {@link net.sf.ehcache.store.DiskStore}.
+     * Newly created caches do not have a {@link net.sf.ehcache.store.MemoryStore} or a {@link net.sf.ehcache.store.disk.DiskStore}.
      * <p/>
      * This method creates those and makes the cache ready to accept elements
      */
@@ -940,10 +1082,10 @@ public interface Ehcache extends Cloneable {
      * @see #isDisabled()
      */
     public void setDisabled(boolean disabled);
-    
+
     /**
      * Returns true if statistics collection is enabled
-     * 
+     *
      * @return true if statistics is enabled, false otherwise
      */
     public boolean isStatisticsEnabled();
@@ -956,30 +1098,30 @@ public interface Ehcache extends Cloneable {
      * parameter <tt>true</tt>.
      * Disabling statistics also disables the sampled statistics collection if
      * it is enabled
-     * 
+     *
      * @param enableStatistics
      */
     public void setStatisticsEnabled(boolean enableStatistics);
 
     /**
      * Returns sampled statistics for this cache.
-     * 
+     *
      * @return The sampled cache statistics
      */
     public SampledCacheStatistics getSampledCacheStatistics();
-    
+
     /**
      * Enable/disable sampled statistics collection.
      * Enabling sampled statistics also enables the normal statistics collection if its not already enabled.
      * Disabling sampled statistics does not have any effect on normal statistics.
-     * 
+     *
      * @param enableStatistics
      */
     public void setSampledStatisticsEnabled(boolean enableStatistics);
 
     /**
      * Returns if sampled statistics collection is enabled or disabled
-     * 
+     *
      * @return true if sampled statistics is enabled, false otherwise
      */
     public boolean isSampledStatisticsEnabled();
@@ -1004,24 +1146,28 @@ public interface Ehcache extends Cloneable {
      * @return the writer manager that's set up for this cache
      */
     public CacheWriterManager getWriterManager();
-    
+
     /**
      * Returns true if the cache is in coherent mode cluster-wide. Returns false otherwise.
      * <p />
      * It applies to coherent clustering mechanisms only e.g. Terracotta
-     * 
+     *
      * @return true if the cache is in coherent mode cluster-wide, false otherwise
+     * @deprecated Use {@link #isClusterBulkLoadEnabled()} instead
      */
-    public boolean isClusterCoherent();
-    
+    @Deprecated
+    public boolean isClusterCoherent() throws TerracottaNotRunningException;
+
     /**
      * Returns true if the cache is in coherent mode for the current node. Returns false otherwise.
      * <p />
      * It applies to coherent clustering mechanisms only e.g. Terracotta
-     * 
+     *
      * @return true if the cache is in coherent mode cluster-wide, false otherwise
+     * @deprecated Use {@link #isNodeBulkLoadEnabled()} instead
      */
-    public boolean isNodeCoherent();
+    @Deprecated
+    public boolean isNodeCoherent() throws TerracottaNotRunningException;
 
     /**
      * Sets the cache in coherent or incoherent mode depending on the parameter on this node.
@@ -1029,39 +1175,200 @@ public interface Ehcache extends Cloneable {
      * calling {@code setNodeCoherent(false)} when already in incoherent mode will be a no-op.
      * <p />
      * It applies to coherent clustering mechanisms only e.g. Terracotta
-     * 
+     * <p />
+     * When using Terracotta clustered caches with nonstop enabled, the timeout used by this method is
+     * {@link NonstopConfiguration#getBulkOpsTimeoutMultiplyFactor()} times the timeout value in the config.
+     *
      * @param coherent
      *            true transitions to coherent mode, false to incoherent mode
      * @throws UnsupportedOperationException if this cache does not support coherence, like RMI replication
+     * @deprecated Use {@link #setNodeBulkLoadEnabled(boolean)} instead
      */
-    public void setNodeCoherent(boolean coherent) throws UnsupportedOperationException;
+    @Deprecated
+    public void setNodeCoherent(boolean coherent) throws UnsupportedOperationException, TerracottaNotRunningException;
 
     /**
-     * This method waits until the cache is in coherent mode in all the connected nodes. If the cache is already in coherent mode it returns
-     * immediately
+     * This method waits until the cache is in coherent mode in all the connected nodes.
+     * If the cache is already in coherent mode it returns immediately
      * <p />
      * It applies to coherent clustering mechanisms only e.g. Terracotta
      * @throws UnsupportedOperationException if this cache does not support coherence, like RMI replication
+     * @deprecated Use {@link #waitUntilClusterBulkLoadComplete()} instead
      */
-    public void waitUntilClusterCoherent() throws UnsupportedOperationException;
+    @Deprecated
+    public void waitUntilClusterCoherent() throws UnsupportedOperationException, TerracottaNotRunningException;
 
     /**
      * This class is used to access the transaction manager used during XA.
      * @param transactionManagerLookup
      */
     public void setTransactionManagerLookup(TransactionManagerLookup transactionManagerLookup);
-    
+
     /**
      * Add a PropertyChangeListener.
-     * 
+     *
      * @param listener
      */
     public void addPropertyChangeListener(PropertyChangeListener listener);
-    
+
     /**
      * Remove a PropertyChangeListener.
-     * 
+     *
      * @param listener
      */
     public void removePropertyChangeListener(PropertyChangeListener listener);
+
+    /**
+     * Retrieve the given named search attribute
+     *
+     * @param <T>
+     *            type of the attribute
+     * @param attributeName
+     *            the name of the attribute to retrieve
+     * @throws CacheException
+     *             if no such attribute is defined for the given name
+     * @return the search attribute
+     */
+    public <T> Attribute<T> getSearchAttribute(String attributeName) throws CacheException;
+
+    /**
+     * Create a new query builder for this cache
+     *
+     * @return a new query builder
+     */
+    public Query createQuery();
+
+    /**
+     * Is this cache searchable?
+     *
+     * @return true if this cache is searchable
+     */
+    public boolean isSearchable();
+
+    /**
+     * Get the average search execution time (in millis) for searches that have completed in the last sample period
+     */
+    public long getAverageSearchTime();
+
+    /**
+     * Get the number of search executions that have completed in the last second
+     */
+    public long getSearchesPerSecond();
+
+    /**
+     * Acquires the proper read lock for a given cache key
+     *
+     * @param key - The key that retrieves a value that you want to protect via locking
+     */
+    public void acquireReadLockOnKey(Object key);
+
+    /**
+     * Acquires the proper write lock for a given cache key
+     *
+     * @param key - The key that retrieves a value that you want to protect via locking
+     */
+    public void acquireWriteLockOnKey(Object key);
+
+    /**
+     * Try to get a read lock on a given key. If can't get it in timeout millis then
+     * return a boolean telling that it didn't get the lock
+     *
+     * @param key - The key that retrieves a value that you want to protect via locking
+     * @param timeout - millis until giveup on getting the lock
+     * @return whether the lock was awarded
+     * @throws InterruptedException
+     */
+    public boolean tryReadLockOnKey(Object key, long timeout) throws InterruptedException;
+
+    /**
+     * Try to get a write lock on a given key. If can't get it in timeout millis then
+     * return a boolean telling that it didn't get the lock
+     *
+     * @param key - The key that retrieves a value that you want to protect via locking
+     * @param timeout - millis until giveup on getting the lock
+     * @return whether the lock was awarded
+     * @throws InterruptedException
+     */
+    public boolean tryWriteLockOnKey(Object key, long timeout) throws InterruptedException;
+
+    /**
+     * Release a held read lock for the passed in key
+     *
+     * @param key - The key that retrieves a value that you want to protect via locking
+     */
+    public void releaseReadLockOnKey(Object key);
+
+    /**
+     * Release a held write lock for the passed in key
+     *
+     * @param key - The key that retrieves a value that you want to protect via locking
+     */
+    public void releaseWriteLockOnKey(Object key);
+
+    /**
+     * Returns true if a read lock for the key is held by the current thread
+     *
+     * @param key
+     * @return true if a read lock for the key is held by the current thread
+     * @throws UnsupportedOperationException if querying the read lock state is not supported
+     */
+    boolean isReadLockedByCurrentThread(Object key) throws UnsupportedOperationException;
+
+    /**
+     * Returns true if a write lock for the key is held by the current thread
+     *
+     * @param key
+     * @return true if a write lock for the key is held by the current thread
+     * @throws UnsupportedOperationException if querying the write lock state is not supported
+     */
+    boolean isWriteLockedByCurrentThread(Object key) throws UnsupportedOperationException;
+
+    /**
+     * Returns true if at least one node in the cluster is in bulk-load mode. Returns false otherwise.
+     * <p />
+     * NOTE: if {@link #isNodeBulkLoadEnabled()} returns true, this method will always return true.
+     * Applies to caches clustered with Terracotta only.
+     *
+     * @throws UnsupportedOperationException if the cache is not clustered with Terracotta
+     * @return true if the cache is in bulk-load mode cluster-wide, false otherwise
+     */
+    public boolean isClusterBulkLoadEnabled() throws UnsupportedOperationException, TerracottaNotRunningException;
+
+    /**
+     * Returns true if the current node is in bulk-load mode. Returns false otherwise.
+     * <p />
+     * NOTE: if this method returns true, {@link #isClusterBulkLoadEnabled()} method will always return true.
+     * Applies to caches clustered with Terracotta only.
+     *
+     * @throws UnsupportedOperationException if the cache is not clustered with Terracotta
+     * @return true if the cache is in coherent mode cluster-wide, false otherwise
+     */
+    public boolean isNodeBulkLoadEnabled() throws UnsupportedOperationException, TerracottaNotRunningException;
+
+    /**
+     * Enable/disable bulk-load mode in this node for this cache.
+     * Calling {@code setBulkLoadEnabled(true)} when the cache is already in bulk-load mode or
+     * calling {@code setBulkLoadEnabled(false)} when already NOT in bulk-load mode will be a no-op.
+     * <p />
+     * Applies to caches clustered with Terracotta only.
+     * <p />
+     * When using Terracotta clustered caches with nonstop enabled, the timeout used by this method is
+     * {@link NonstopConfiguration#getBulkOpsTimeoutMultiplyFactor()} times the timeout value in the nonstop config.
+     *
+     * @param enabledBulkLoad
+     *            true enables bulk-load, false disables it if not already disabled
+     * @throws UnsupportedOperationException if the cache is not clustered with Terracotta
+     */
+    public void setNodeBulkLoadEnabled(boolean enabledBulkLoad) throws UnsupportedOperationException, TerracottaNotRunningException;
+
+    /**
+     * This method waits until all the connected nodes have disabled bulk-load. Or in other words, calling this method
+     * will block until all connected nodes in the cluster disables bulk-load. If none of the nodes did not enable bulk-load
+     * this method will return immediately
+     * <p />
+     * Applies to caches clustered with Terracotta only.
+     * @throws UnsupportedOperationException if the cache is not clustered with Terracotta
+     */
+    public void waitUntilClusterBulkLoadComplete() throws UnsupportedOperationException, TerracottaNotRunningException;
+
 }
