@@ -25,7 +25,7 @@ import net.sf.ehcache.writer.writethrough.WriteThroughManager;
  * Class to hold the CacheWriterManager configuration
  *
  * @author Geert Bevin
- * @version $Id: CacheWriterConfiguration.java 2154 2010-04-06 02:45:52Z cdennis $
+ * @version $Id: CacheWriterConfiguration.java 3883 2011-04-09 18:17:18Z hhuynh $
  */
 public class CacheWriterConfiguration implements Cloneable {
     /**
@@ -69,6 +69,16 @@ public class CacheWriterConfiguration implements Cloneable {
      */
     public static final int DEFAULT_RETRY_ATTEMPT_DELAY_SECONDS = 1;
 
+    /**
+     * Default concurrency level for write behind
+     */
+    public static final int DEFAULT_WRITE_BEHIND_CONCURRENCY = 1;
+
+    /**
+     * Default max queue size for write behind
+     */
+    public static final int DEFAULT_WRITE_BEHIND_MAX_QUEUE_SIZE = 0;
+
     /**
      * Represents how elements are written to the {@link net.sf.ehcache.writer.CacheWriter}
      */
@@ -118,6 +128,8 @@ public class CacheWriterConfiguration implements Cloneable {
     private int writeBatchSize = DEFAULT_WRITE_BATCH_SIZE;
     private int retryAttempts = DEFAULT_RETRY_ATTEMPTS;
     private int retryAttemptDelaySeconds = DEFAULT_RETRY_ATTEMPT_DELAY_SECONDS;
+    private int writeBehindConcurrency = DEFAULT_WRITE_BEHIND_CONCURRENCY;
+    private int writeBehindMaxQueueSize = DEFAULT_WRITE_BEHIND_MAX_QUEUE_SIZE;
     private CacheWriterFactoryConfiguration cacheWriterFactoryConfiguration;
 
     /**
@@ -481,6 +493,22 @@ public class CacheWriterConfiguration implements Cloneable {
      * Configuration for the CacheWriterFactoryConfiguration.
      */
     public static final class CacheWriterFactoryConfiguration extends FactoryConfiguration<CacheWriterFactoryConfiguration> {
+        
+        /**
+         * Overrided hashCode()
+         */
+        @Override
+        public int hashCode() {
+            return super.hashCode();
+        }
+
+        /**
+         * Overrided equals
+         */
+        @Override
+        public boolean equals(Object obj) {
+            return super.equals(obj);
+        }
     }
 
     /**
@@ -507,4 +535,153 @@ public class CacheWriterConfiguration implements Cloneable {
     public CacheWriterFactoryConfiguration getCacheWriterFactoryConfiguration() {
         return cacheWriterFactoryConfiguration;
     }
-}
\ No newline at end of file
+
+    /**
+     * Configures the amount of thread/bucket pairs WriteBehind should use
+     * @param concurrency Amount of thread/bucket pairs, has to be at least 1
+     */
+    public void setWriteBehindConcurrency(int concurrency) {
+        if (concurrency < 1) {
+            this.writeBehindConcurrency = 1;
+        } else {
+            this.writeBehindConcurrency = concurrency;
+        }
+    }
+
+    /**
+     *
+     * @param concurrency Amount of thread/bucket pairs, has to be at least 1
+     * @return this configuration instance
+     * @see #setWriteBehindConcurrency(int)
+     */
+    public CacheWriterConfiguration writeBehindConcurrency(int concurrency) {
+        this.setWriteBehindConcurrency(concurrency);
+        return this;
+    }
+
+    /**
+     * Accessor
+     * @return the amount of bucket/thread pairs configured for this cache's write behind
+     */
+    public int getWriteBehindConcurrency() {
+        return writeBehindConcurrency;
+    }
+
+    /**
+     * Configures the maximum amount of operations to be on the waiting queue, before it blocks
+     * @param writeBehindMaxQueueSize maximum amount of operations allowed on the waiting queue
+     */
+    public void setWriteBehindMaxQueueSize(final int writeBehindMaxQueueSize) {
+        if (writeBehindMaxQueueSize < 0) {
+            this.writeBehindMaxQueueSize = DEFAULT_WRITE_BEHIND_MAX_QUEUE_SIZE;
+        } else {
+            this.writeBehindMaxQueueSize = writeBehindMaxQueueSize;
+        }
+    }
+
+    /**
+     * Accessor
+     * @return the maximum amount of operations allowed on the write behind queue
+     */
+    public int getWriteBehindMaxQueueSize() {
+        return writeBehindMaxQueueSize;
+    }
+
+    /**
+     * @param writeBehindMaxQueueSize maximum amount of operations allowed on the waiting queue
+     * @return this configuration instance
+     * @see #setWriteBehindMaxQueueSize(int)
+     */
+    public CacheWriterConfiguration writeBehindMaxQueueSize(int writeBehindMaxQueueSize) {
+        this.setWriteBehindMaxQueueSize(writeBehindMaxQueueSize);
+        return this;
+    }
+
+    /**
+     * Overrided hashCode()
+     */
+    @Override
+    public int hashCode() {
+        final int prime = 31;
+        final int primeTwo = 1231;
+        final int primeThree = 1237;
+        int result = 1;
+        result = prime * result + ((cacheWriterFactoryConfiguration == null) ? 0 : cacheWriterFactoryConfiguration.hashCode());
+        result = prime * result + maxWriteDelay;
+        result = prime * result + minWriteDelay;
+        result = prime * result + (notifyListenersOnException ? primeTwo : primeThree);
+        result = prime * result + rateLimitPerSecond;
+        result = prime * result + retryAttemptDelaySeconds;
+        result = prime * result + retryAttempts;
+        result = prime * result + writeBatchSize;
+        result = prime * result + (writeBatching ? primeTwo : primeThree);
+        result = prime * result + (writeCoalescing ? primeTwo : primeThree);
+        result = prime * result + ((writeMode == null) ? 0 : writeMode.hashCode());
+        result = prime * result + writeBehindConcurrency;
+        return result;
+    }
+
+    /**
+     * Overrided equals()
+     */
+    @Override
+    public boolean equals(Object obj) {
+        if (this == obj) {
+            return true;
+        }
+        if (obj == null) {
+            return false;
+        }
+        if (getClass() != obj.getClass()) {
+            return false;
+        }
+        CacheWriterConfiguration other = (CacheWriterConfiguration) obj;
+        if (cacheWriterFactoryConfiguration == null) {
+            if (other.cacheWriterFactoryConfiguration != null) {
+                return false;
+            }
+        } else if (!cacheWriterFactoryConfiguration.equals(other.cacheWriterFactoryConfiguration)) {
+            return false;
+        }
+        if (maxWriteDelay != other.maxWriteDelay) {
+            return false;
+        }
+        if (minWriteDelay != other.minWriteDelay) {
+            return false;
+        }
+        if (notifyListenersOnException != other.notifyListenersOnException) {
+            return false;
+        }
+        if (rateLimitPerSecond != other.rateLimitPerSecond) {
+            return false;
+        }
+        if (retryAttemptDelaySeconds != other.retryAttemptDelaySeconds) {
+            return false;
+        }
+        if (retryAttempts != other.retryAttempts) {
+            return false;
+        }
+        if (writeBatchSize != other.writeBatchSize) {
+            return false;
+        }
+        if (writeBatching != other.writeBatching) {
+            return false;
+        }
+        if (writeCoalescing != other.writeCoalescing) {
+            return false;
+        }
+        if (writeBehindConcurrency != other.writeBehindConcurrency) {
+            return false;
+        }
+        if (writeMode == null) {
+            if (other.writeMode != null) {
+                return false;
+            }
+        } else if (!writeMode.equals(other.writeMode)) {
+            return false;
+        }
+        return true;
+    }
+    
+    
+}
