@@ -34,9 +34,13 @@ import net.sf.ehcache.event.RegisteredEventListeners;
 import net.sf.ehcache.exceptionhandler.CacheExceptionHandler;
 import net.sf.ehcache.extension.CacheExtension;
 import net.sf.ehcache.loader.CacheLoader;
+import net.sf.ehcache.search.Attribute;
+import net.sf.ehcache.search.Query;
 import net.sf.ehcache.statistics.CacheUsageListener;
 import net.sf.ehcache.statistics.LiveCacheStatistics;
 import net.sf.ehcache.statistics.sampled.SampledCacheStatistics;
+import net.sf.ehcache.terracotta.InternalEhcache;
+import net.sf.ehcache.terracotta.TerracottaNotRunningException;
 import net.sf.ehcache.transaction.manager.TransactionManagerLookup;
 import net.sf.ehcache.writer.CacheWriter;
 import net.sf.ehcache.writer.CacheWriterManager;
@@ -45,11 +49,11 @@ import net.sf.ehcache.writer.CacheWriterManager;
  * Adapter class for Ehcache interface decorators. Implements all method in {@link Ehcache} by delegating all calls to the decorated
  * {@link Ehcache}. This class is provided as a convenience for easily creating {@link Ehcache} decorators by extending this class and
  * overriding only the methods of interest.
- * 
+ *
  * @author Abhishek Sanoujam
- * 
+ *
  */
-public class EhcacheDecoratorAdapter implements Ehcache {
+public class EhcacheDecoratorAdapter implements InternalEhcache {
 
     /**
      * The decorated {@link Ehcache}, has protected visibility so that sub-classes can have access to it.
@@ -58,7 +62,7 @@ public class EhcacheDecoratorAdapter implements Ehcache {
 
     /**
      * Constructor accepting the cache to be decorated
-     * 
+     *
      * @param underlyingCache
      */
     public EhcacheDecoratorAdapter(Ehcache underlyingCache) {
@@ -72,6 +76,13 @@ public class EhcacheDecoratorAdapter implements Ehcache {
         return underlyingCache.get(key);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public Map<Object, Element> getAll(Collection<?> keys) throws IllegalStateException, CacheException {
+        return underlyingCache.getAll(keys);
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -108,6 +119,13 @@ public class EhcacheDecoratorAdapter implements Ehcache {
         underlyingCache.put(element);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public void putAll(Collection<Element> elements) throws IllegalArgumentException, IllegalStateException, CacheException {
+        underlyingCache.putAll(elements);
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -136,6 +154,20 @@ public class EhcacheDecoratorAdapter implements Ehcache {
         return underlyingCache.remove(key);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public void removeAll(Collection<?> keys) throws IllegalStateException {
+        underlyingCache.removeAll(keys);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void removeAll(Collection<?> keys, boolean doNotNotifyCacheReplicators) throws IllegalStateException {
+        underlyingCache.removeAll(keys, doNotNotifyCacheReplicators);
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -178,6 +210,27 @@ public class EhcacheDecoratorAdapter implements Ehcache {
         return underlyingCache.calculateInMemorySize();
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public long calculateOffHeapSize() throws IllegalStateException, CacheException {
+        return underlyingCache.calculateOffHeapSize();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public long calculateOnDiskSize() throws IllegalStateException, CacheException {
+        return underlyingCache.calculateOnDiskSize();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public boolean hasAbortedSizeOf() {
+        return underlyingCache.hasAbortedSizeOf();
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -287,7 +340,7 @@ public class EhcacheDecoratorAdapter implements Ehcache {
      * {@inheritDoc}
      */
     public CacheExceptionHandler getCacheExceptionHandler() {
-        return this.getCacheExceptionHandler();
+        return underlyingCache.getCacheExceptionHandler();
     }
 
     /**
@@ -297,6 +350,13 @@ public class EhcacheDecoratorAdapter implements Ehcache {
         return underlyingCache.getCacheManager();
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public long getOffHeapStoreSize() throws IllegalStateException {
+        return underlyingCache.getOffHeapStoreSize();
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -439,7 +499,10 @@ public class EhcacheDecoratorAdapter implements Ehcache {
 
     /**
      * {@inheritDoc}
+     *
+     * @deprecated Use {@link #isClusterBulkLoadEnabled()} instead
      */
+    @Deprecated
     public boolean isClusterCoherent() {
         return underlyingCache.isClusterCoherent();
     }
@@ -495,7 +558,10 @@ public class EhcacheDecoratorAdapter implements Ehcache {
 
     /**
      * {@inheritDoc}
+     *
+     * @deprecated Use {@link #isNodeBulkLoadEnabled()} instead
      */
+    @Deprecated
     public boolean isNodeCoherent() {
         return underlyingCache.isNodeCoherent();
     }
@@ -614,7 +680,10 @@ public class EhcacheDecoratorAdapter implements Ehcache {
 
     /**
      * {@inheritDoc}
+     *
+     * @deprecated Use {@link #setNodeBulkLoadEnabled(boolean)} instead
      */
+    @Deprecated
     public void setNodeCoherent(boolean coherent) throws UnsupportedOperationException {
         underlyingCache.setNodeCoherent(coherent);
     }
@@ -663,7 +732,10 @@ public class EhcacheDecoratorAdapter implements Ehcache {
 
     /**
      * {@inheritDoc}
+     *
+     * @deprecated Use {@link #waitUntilClusterBulkLoadComplete()} instead
      */
+    @Deprecated
     public void waitUntilClusterCoherent() throws UnsupportedOperationException {
         underlyingCache.waitUntilClusterCoherent();
     }
@@ -703,10 +775,10 @@ public class EhcacheDecoratorAdapter implements Ehcache {
     public Element replace(Element element) throws NullPointerException {
         return underlyingCache.replace(element);
     }
-    
+
     /**
      * {@inheritDoc}
-     * 
+     *
      * @see net.sf.ehcache.Ehcache#addPropertyChangeListener(java.beans.PropertyChangeListener)
      */
     public void addPropertyChangeListener(PropertyChangeListener listener) {
@@ -715,11 +787,184 @@ public class EhcacheDecoratorAdapter implements Ehcache {
 
     /**
      * {@inheritDoc}
-     * 
+     *
      * @see net.sf.ehcache.Ehcache#removePropertyChangeListener(java.beans.PropertyChangeListener)
      */
     public void removePropertyChangeListener(PropertyChangeListener listener) {
         underlyingCache.addPropertyChangeListener(listener);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public String toString() {
+        return underlyingCache.toString();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public Query createQuery() {
+        return underlyingCache.createQuery();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public <T> Attribute<T> getSearchAttribute(String attributeName) throws CacheException {
+        return underlyingCache.getSearchAttribute(attributeName);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public boolean isSearchable() {
+        return underlyingCache.isSearchable();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public long getAverageSearchTime() {
+        return underlyingCache.getAverageSearchTime();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public long getSearchesPerSecond() {
+        return underlyingCache.getSearchesPerSecond();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void acquireReadLockOnKey(Object key) {
+        underlyingCache.acquireReadLockOnKey(key);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void acquireWriteLockOnKey(Object key) {
+        underlyingCache.acquireWriteLockOnKey(key);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void releaseReadLockOnKey(Object key) {
+        underlyingCache.releaseReadLockOnKey(key);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void releaseWriteLockOnKey(Object key) {
+        underlyingCache.releaseWriteLockOnKey(key);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public boolean tryReadLockOnKey(Object key, long timeout) throws InterruptedException {
+        return underlyingCache.tryReadLockOnKey(key, timeout);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public boolean tryWriteLockOnKey(Object key, long timeout) throws InterruptedException {
+        return underlyingCache.tryWriteLockOnKey(key, timeout);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public boolean isReadLockedByCurrentThread(Object key) {
+        return underlyingCache.isReadLockedByCurrentThread(key);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public boolean isWriteLockedByCurrentThread(Object key) {
+        return underlyingCache.isWriteLockedByCurrentThread(key);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public boolean isClusterBulkLoadEnabled() throws UnsupportedOperationException, TerracottaNotRunningException {
+        return underlyingCache.isClusterBulkLoadEnabled();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public boolean isNodeBulkLoadEnabled() throws UnsupportedOperationException, TerracottaNotRunningException {
+        return underlyingCache.isNodeBulkLoadEnabled();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void setNodeBulkLoadEnabled(boolean enabledBulkLoad) throws UnsupportedOperationException, TerracottaNotRunningException {
+        underlyingCache.setNodeBulkLoadEnabled(enabledBulkLoad);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void waitUntilClusterBulkLoadComplete() throws UnsupportedOperationException, TerracottaNotRunningException {
+        underlyingCache.waitUntilClusterBulkLoadComplete();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void unpinAll() {
+        underlyingCache.unpinAll();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public boolean isPinned(Object key) {
+        return underlyingCache.isPinned(key);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void setPinned(Object key, boolean pinned) {
+        underlyingCache.setPinned(key, pinned);
+    }
+
+    private InternalEhcache asInternalEhcache() {
+        return (InternalEhcache) underlyingCache;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public Element removeAndReturnElement(Object key) throws IllegalStateException {
+        if (underlyingCache instanceof InternalEhcache) {
+            return asInternalEhcache().removeAndReturnElement(key);
+        } else {
+            throw new UnsupportedOperationException();
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void recalculateSize(Object key) {
+        if (underlyingCache instanceof InternalEhcache) {
+            asInternalEhcache().recalculateSize(key);
+        } else {
+            throw new UnsupportedOperationException();
+        }
+    }
 }
