@@ -66,7 +66,7 @@ import org.slf4j.LoggerFactory;
  * has a shutdown hook which calls dispose() as a convenience feature for developers.
  *
  * @author Greg Luck
- * @version $Id: RMICacheManagerPeerListener.java 2154 2010-04-06 02:45:52Z cdennis $
+ * @version $Id: RMICacheManagerPeerListener.java 3662 2011-02-18 18:27:20Z lorban $
  */
 public class RMICacheManagerPeerListener implements CacheManagerPeerListener {
 
@@ -274,7 +274,12 @@ public class RMICacheManagerPeerListener implements CacheManagerPeerListener {
             synchronized (cachePeers) {
                 if (cachePeers.get(name) == null) {
                     if (isDistributed(cache)) {
-                        RMICachePeer peer = new RMICachePeer(cache, hostName, port, remoteObjectPort, socketTimeoutMillis);
+                        RMICachePeer peer;
+                        if (cache.getCacheConfiguration().getTransactionalMode().isTransactional()) {
+                            peer = new TransactionalRMICachePeer(cache, hostName, port, remoteObjectPort, socketTimeoutMillis);
+                        } else {
+                            peer = new RMICachePeer(cache, hostName, port, remoteObjectPort, socketTimeoutMillis);
+                        }
                         cachePeers.put(name, peer);
                     }
                 }
@@ -522,7 +527,11 @@ public class RMICacheManagerPeerListener implements CacheManagerPeerListener {
             RMICachePeer rmiCachePeer = null;
             String url = null;
             try {
-                rmiCachePeer = new RMICachePeer(cache, hostName, port, remoteObjectPort, socketTimeoutMillis);
+                if (cache.getCacheConfiguration().getTransactionalMode().isTransactional()) {
+                    rmiCachePeer = new TransactionalRMICachePeer(cache, hostName, port, remoteObjectPort, socketTimeoutMillis);
+                } else {
+                    rmiCachePeer = new RMICachePeer(cache, hostName, port, remoteObjectPort, socketTimeoutMillis);
+                }
                 url = rmiCachePeer.getUrl();
                 bind(url, rmiCachePeer);
             } catch (Exception e) {
