@@ -42,10 +42,11 @@ import java.util.logging.Logger;
  * An implementation of write behind with a queue that is kept in non durable local heap.
  *
  * @author Geert Bevin
- * @version $Id: WriteBehindQueue.java 2379 2010-05-03 15:16:18Z gbevin $
+ * @version $Id: WriteBehindQueue.java 4467 2011-08-02 10:07:08Z alexsnaps $
  */
-public class WriteBehindQueue implements WriteBehind {
-    private final static Logger LOGGER = Logger.getLogger(WriteBehindQueue.class.getName());
+class WriteBehindQueue {
+  
+    private static final Logger LOGGER = Logger.getLogger(WriteBehindQueue.class.getName());
 
     private static final int MS_IN_SEC = 1000;
 
@@ -53,6 +54,7 @@ public class WriteBehindQueue implements WriteBehind {
     private final long minWriteDelayMs;
     private final long maxWriteDelayMs;
     private final int rateLimitPerSecond;
+    private final int maxQueueSize;
     private final boolean writeBatching;
     private final int writeBatchSize;
     private final int retryAttempts;
@@ -62,6 +64,7 @@ public class WriteBehindQueue implements WriteBehind {
     private final ReentrantReadWriteLock queueLock = new ReentrantReadWriteLock();
     private final ReentrantReadWriteLock.ReadLock queueReadLock = queueLock.readLock();
     private final ReentrantReadWriteLock.WriteLock queueWriteLock = queueLock.writeLock();
+    private final Condition queueIsFull = queueWriteLock.newCondition();
     private final Condition queueIsEmpty = queueWriteLock.newCondition();
     private final Condition queueIsStopped = queueWriteLock.newCondition();
 
@@ -92,6 +95,7 @@ public class WriteBehindQueue implements WriteBehind {
         this.minWriteDelayMs = cacheWriterConfig.getMinWriteDelay() * MS_IN_SEC;
         this.maxWriteDelayMs = cacheWriterConfig.getMaxWriteDelay() * MS_IN_SEC;
         this.rateLimitPerSecond = cacheWriterConfig.getRateLimitPerSecond();
+        this.maxQueueSize = cacheWriterConfig.getWriteBehindMaxQueueSize();
         this.writeBatching = cacheWriterConfig.getWriteBatching();
         this.writeBatchSize = cacheWriterConfig.getWriteBatchSize();
         this.retryAttempts = cacheWriterConfig.getRetryAttempts();
@@ -143,9 +147,9 @@ public class WriteBehindQueue implements WriteBehind {
         public void run() {
             try {
                 while (!isStopped()) {
-    
+
                     processItems();
-                    
+
                     queueWriteLock.lock();
                     try {
                         // Wait for new items or until the min write delay has expired.
@@ -154,26 +158,33 @@ public class WriteBehindQueue implements WriteBehind {
                         // the item processor is very fast, causing a large amount of data churn.
                         // However, if the write delay is expired, the processing should start immediately.
                         try {
-                            long delay = minWriteDelayMs;
-                            do {
-                                queueIsEmpty.await(delay, TimeUnit.MILLISECONDS);
-                                long actualDelay = System.currentTimeMillis() - getLastProcessing();
-                                if (actualDelay < minWriteDelayMs) {
-                                    delay = minWriteDelayMs - actualDelay;
-                                } else {
-                                    delay = 0;
+                            if (minWriteDelayMs != 0) {
+                                long delay = minWriteDelayMs;
+                                do {
+                                    queueIsEmpty.await(delay, TimeUnit.MILLISECONDS);
+                                    long actualDelay = System.currentTimeMillis() - getLastProcessing();
+                                    if (actualDelay < minWriteDelayMs) {
+                                        delay = minWriteDelayMs - actualDelay;
+                                    } else {
+                                        delay = 0;
+                                    }
+                                } while (delay > 0);
+                            } else {
+                                while (!stopping && waiting.size() == 0) {
+                                    queueIsEmpty.await();
                                 }
-                            } while (delay > 0);
+                            }
                         } catch (final InterruptedException e) {
                             // if the wait for items is interrupted, act as if the bucket was cancelled
                             stop();
                             Thread.currentThread().interrupt();
                         }
-                        
+
                         // If the queue is stopping and no more work is outstanding, perform the actual stop operation
                         if (stopping && waiting.isEmpty()) {
                             stopTheQueueThread();
-                        }                        
+                        }
+                        queueIsFull.signal();
                     } finally {
                         queueWriteLock.unlock();
                     }
@@ -364,7 +375,9 @@ public class WriteBehindQueue implements WriteBehind {
                     break;
                 } catch (final RuntimeException e) {
                     if (executionsLeft <= 0) {
-                        throw e;
+                        for (SingleOperation singleOperation : itemsPerType) {
+                            singleOperation.throwAway(cacheWriter, e);
+                        }
                     } else {
                         LOGGER.warning("Exception while processing write behind queue, retrying in " + retryAttemptDelaySeconds
                                 + " seconds, " + executionsLeft + " retries left : " + e.getMessage());
@@ -404,7 +417,12 @@ public class WriteBehindQueue implements WriteBehind {
                     break;
                 } catch (final RuntimeException e) {
                     if (executionsLeft <= 0) {
-                        throw e;
+                        try {
+                            item.throwAway(cacheWriter, e);
+                        } catch (RuntimeException runtimeException) {
+                            // TODO Really only log ?!
+                            // LOGGER.warning("Throwing key '" + item.getKey() + "' away triggered an Exception!", runtimeException);
+                        }
                     } else {
                         LOGGER.warning("Exception while processing write behind queue, retrying in " + retryAttemptDelaySeconds
                                 + " seconds, " + executionsLeft + " retries left : " + e.getMessage());
@@ -428,28 +446,49 @@ public class WriteBehindQueue implements WriteBehind {
     public void write(Element element) {
         queueWriteLock.lock();
         try {
+            waitForQueueSizeToDrop();
             if (stopping || stopped) {
                 throw new CacheException("The element '" + element + "' couldn't be added through the write-behind queue for cache '"
                         + cacheName + "' since it's not started.");
             }
             waiting.add(new WriteOperation(element));
+            if (waiting.size() + 1 < maxQueueSize) {
+                queueIsFull.signal();
+            }
             queueIsEmpty.signal();
         } finally {
             queueWriteLock.unlock();
         }
     }
 
+    private void waitForQueueSizeToDrop() {
+        if (maxQueueSize > 0) {
+            while (getQueueSize() >= maxQueueSize) {
+                try {
+                    queueIsFull.await();
+                } catch (InterruptedException e) {
+                    stop();
+                    Thread.currentThread().interrupt();
+                }
+            }
+        }
+    }
+
     /**
      * {@inheritDoc}
      */
     public void delete(CacheEntry entry) {
         queueWriteLock.lock();
         try {
+            waitForQueueSizeToDrop();
             if (stopping || stopped) {
-                throw new CacheException("The entry for key '" + entry.getKey() + "' couldn't be deleted through the write-behind " 
+                throw new CacheException("The entry for key '" + entry.getKey() + "' couldn't be deleted through the write-behind "
                         + "queue for cache '" + cacheName + "' since it's not started.");
             }
             waiting.add(new DeleteOperation(entry));
+            if (waiting.size() + 1 < maxQueueSize) {
+                queueIsFull.signal();
+            }
             queueIsEmpty.signal();
         } finally {
             queueWriteLock.unlock();
@@ -465,7 +504,7 @@ public class WriteBehindQueue implements WriteBehind {
             if (stopped) {
                 return;
             }
-            
+
             stopping = true;
             queueIsEmpty.signal();
             while (!stopped) {
@@ -479,6 +518,15 @@ public class WriteBehindQueue implements WriteBehind {
         }
     }
 
+    /**
+     * Gets the best estimate for items in the queue still awaiting processing.
+     * Not including elements currently processed
+     * @return the amount of elements still awaiting processing.
+     */
+    public long getQueueSize() {
+        return waiting.size();
+    }
+
     private boolean isStopped() {
         queueReadLock.lock();
         try {
