@@ -16,13 +16,19 @@
 
 package net.sf.ehcache;
 
+import static org.hamcrest.CoreMatchers.equalTo;
+import static org.hamcrest.CoreMatchers.is;
+import static org.hamcrest.CoreMatchers.notNullValue;
+import static org.hamcrest.CoreMatchers.nullValue;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertThat;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
+import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.InputStream;
@@ -32,24 +38,31 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
 
+import junit.framework.Assert;
 import net.sf.ehcache.bootstrap.BootstrapCacheLoader;
 import net.sf.ehcache.config.CacheConfiguration;
 import net.sf.ehcache.config.Configuration;
 import net.sf.ehcache.config.ConfigurationFactory;
 import net.sf.ehcache.config.DiskStoreConfiguration;
+import net.sf.ehcache.config.InvalidConfigurationException;
+import net.sf.ehcache.config.MemoryUnit;
 import net.sf.ehcache.constructs.blocking.BlockingCache;
 import net.sf.ehcache.constructs.blocking.CountingCacheEntryFactory;
 import net.sf.ehcache.constructs.blocking.SelfPopulatingCache;
+import net.sf.ehcache.distribution.AbstractRMITest;
 import net.sf.ehcache.distribution.JVMUtil;
 import net.sf.ehcache.distribution.RMIAsynchronousCacheReplicator;
 import net.sf.ehcache.distribution.RMIBootstrapCacheLoader;
 import net.sf.ehcache.event.CacheEventListener;
 import net.sf.ehcache.event.RegisteredEventListeners;
 import net.sf.ehcache.statistics.LiveCacheStatisticsData;
-import net.sf.ehcache.store.DiskStore;
 import net.sf.ehcache.store.Store;
+import net.sf.ehcache.store.disk.DiskStore;
+import net.sf.ehcache.util.MemorySizeParser;
 
 import org.junit.After;
+import org.junit.BeforeClass;
+import org.junit.Ignore;
 import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -58,11 +71,12 @@ import org.slf4j.LoggerFactory;
  * Tests for CacheManager
  *
  * @author Greg Luck
- * @version $Id: CacheManagerTest.java 2312 2010-04-21 13:47:15Z alexsnaps $
+ * @version $Id: CacheManagerTest.java 4739 2011-09-25 08:25:33Z asingh $
  */
 public class CacheManagerTest {
 
     private static final Logger LOG = LoggerFactory.getLogger(CacheManagerTest.class.getName());
+    private static final int CACHES_IN_EHCACHE_XML = 15;
 
     /**
      * the CacheManager Singleton instance
@@ -74,6 +88,11 @@ public class CacheManagerTest {
      */
     protected CacheManager instanceManager;
 
+    @BeforeClass
+    public static void installRMISocketFactory() {
+        AbstractRMITest.installRMISocketFactory();
+    }
+
     /**
      * Shutdown managers. Check that the manager is removed from
      * CacheManager.ALL_CACHE_MANAGERS
@@ -96,11 +115,299 @@ public class CacheManagerTest {
         }
     }
 
+    @Test(expected = InvalidConfigurationException.class)
+    public void testCantMixCountAndSizeBasedTunings() {
+        Configuration configuration = new Configuration().maxBytesLocalHeap(16, MemoryUnit.MEGABYTES);
+        CacheManager cacheManager = new CacheManager(configuration);
+        try {
+            cacheManager.addCache(new Cache(new CacheConfiguration("zero", 10)));
+        } finally {
+            cacheManager.shutdown();
+        }
+    }
+
+    @Test
+    public void testReduceCacheManagerPoolBelowReservedUsage() {
+        Configuration configuration = new Configuration()
+            .maxBytesLocalHeap(16, MemoryUnit.MEGABYTES);
+        CacheManager cacheManager = new CacheManager(configuration);
+        cacheManager.addCache(new Cache(new CacheConfiguration("zero", 0)));
+        cacheManager.addCache(new Cache(new CacheConfiguration("one", 0).maxBytesLocalHeap(2, MemoryUnit.MEGABYTES)));
+        cacheManager.addCache(new Cache(new CacheConfiguration("two", 0).maxBytesLocalHeap(2, MemoryUnit.MEGABYTES)));
+        assertThat(cacheManager.getOnHeapPool().getMaxSize(), is(MemoryUnit.MEGABYTES.toBytes(12)));
+        cacheManager.getConfiguration().maxBytesLocalHeap(8, MemoryUnit.MEGABYTES);
+        assertThat(cacheManager.getOnHeapPool().getMaxSize(), is(MemoryUnit.MEGABYTES.toBytes(4)));
+        cacheManager.getConfiguration().maxBytesLocalHeap(4, MemoryUnit.MEGABYTES);
+        assertThat(cacheManager.getOnHeapPool().getMaxSize(), is(MemoryUnit.MEGABYTES.toBytes(0)));
+        try {
+            cacheManager.getConfiguration().maxBytesLocalHeap(3, MemoryUnit.MEGABYTES);
+            fail();
+        } catch (InvalidConfigurationException e) {
+            assertThat(e.getMessage().contains("one"), is(true));
+            assertThat(e.getMessage().contains("two"), is(true));
+            assertThat(e.getMessage().contains("zero"), is(false));
+        }
+        cacheManager.removeCache("one");
+        assertThat(cacheManager.getOnHeapPool().getMaxSize(), is(MemoryUnit.MEGABYTES.toBytes(2)));
+        CacheConfiguration two = cacheManager.getCache("two").getCacheConfiguration();
+        two.maxBytesLocalHeap(1, MemoryUnit.MEGABYTES);
+        assertThat(cacheManager.getOnHeapPool().getMaxSize(), is(MemoryUnit.MEGABYTES.toBytes(3)));
+        cacheManager.removeCache("two");
+        assertThat(cacheManager.getOnHeapPool().getMaxSize(), is(MemoryUnit.MEGABYTES.toBytes(4)));
+        two.maxBytesLocalHeap(2, MemoryUnit.MEGABYTES);
+        assertThat(cacheManager.getOnHeapPool().getMaxSize(), is(MemoryUnit.MEGABYTES.toBytes(4)));
+
+        cacheManager.shutdown();
+    }
+
+    @Test
+    public void testCantAddCacheWhenOverAllocatingCacheManagerPool() {
+        Configuration configuration = new Configuration()
+            .maxBytesLocalHeap(5, MemoryUnit.MEGABYTES);
+        CacheManager cacheManager = new CacheManager(configuration);
+        cacheManager.addCache(new Cache(new CacheConfiguration("one", 0).maxBytesLocalHeap(2, MemoryUnit.MEGABYTES)));
+        assertThat(cacheManager.getOnHeapPool().getMaxSize(), is(MemoryUnit.MEGABYTES.toBytes(3)));
+        cacheManager.addCache(new Cache(new CacheConfiguration("two", 0).maxBytesLocalHeap(2, MemoryUnit.MEGABYTES)));
+        assertThat(cacheManager.getOnHeapPool().getMaxSize(), is(MemoryUnit.MEGABYTES.toBytes(1)));
+        try {
+            cacheManager.addCache(new Cache(new CacheConfiguration("three", 0).maxBytesLocalHeap(2, MemoryUnit.MEGABYTES)));
+        } catch (InvalidConfigurationException e) {
+            assertThat(e.getMessage().contains("'three'"), is(true));
+            assertThat(e.getMessage().contains("over-allocate"), is(true));
+        }
+        assertThat(cacheManager.getCache("three"), nullValue());
+        assertThat(cacheManager.getConfiguration().getCacheConfigurations().get("three"), nullValue());
+        assertThat(cacheManager.getConfiguration().getCacheConfigurations().size(), is(2));
+        assertThat(cacheManager.getCacheNames().length, is(2));
+        cacheManager.shutdown();
+    }
+
+    // todo This should be addressed at some point: we're cloning things around too much...
+    @Ignore
+    @Test
+    public void testCacheConfigurationAreInSync() {
+        Configuration configuration = new Configuration().cache(new CacheConfiguration("one", 0));
+        CacheManager cacheManager = new CacheManager(configuration);
+        assertThat(cacheManager.getCache("one").getCacheConfiguration(),
+                is(cacheManager.getConfiguration().getCacheConfigurations().get("one")));
+    }
+
+    @Test
+    public void testMaxBytesOnCacheDynamicChangesReflectOnPercentBasedCaches() throws Exception {
+        CacheConfiguration configuration1 = new CacheConfiguration("one", 0);
+        CacheConfiguration configuration2 = new CacheConfiguration("two", 0);
+        Configuration configuration = new Configuration()
+            .maxBytesLocalHeap(5, MemoryUnit.MEGABYTES)
+            .cache(configuration1)
+            .cache(configuration2)
+            .cache(new CacheConfiguration("three", 0));
+        configuration1.setMaxBytesLocalHeap("20%");
+        configuration2.setMaxBytesLocalHeap("20%");
+
+        CacheManager cacheManager = new CacheManager(configuration);
+        assertThat(cacheManager.getCache("one").getCacheConfiguration().getMaxBytesLocalHeap(), equalTo(MemoryUnit.MEGABYTES.toBytes(1)));
+        assertThat(cacheManager.getCache("two").getCacheConfiguration().getMaxBytesLocalHeap(), equalTo(MemoryUnit.MEGABYTES.toBytes(1)));
+        assertThat(cacheManager.getCache("three").getCacheConfiguration().getMaxBytesLocalHeap(), equalTo(0L));
+
+        configuration.maxBytesLocalHeap(10, MemoryUnit.MEGABYTES);
+
+        assertThat(cacheManager.getCache("one").getCacheConfiguration().getMaxBytesLocalHeap(), equalTo(MemoryUnit.MEGABYTES.toBytes(2)));
+        assertThat(cacheManager.getCache("two").getCacheConfiguration().getMaxBytesLocalHeap(), equalTo(MemoryUnit.MEGABYTES.toBytes(2)));
+        assertThat(cacheManager.getCache("three").getCacheConfiguration().getMaxBytesLocalHeap(), equalTo(0L));
+
+        cacheManager.shutdown();
+    }
+
+    @Test
+    public void testMaxBytesOnCacheConfiguration() throws Exception {
+        CacheConfiguration configuration1 = new CacheConfiguration("one", 0);
+        CacheConfiguration configuration2 = new CacheConfiguration("two", 0);
+        Configuration configuration = new Configuration()
+            .maxBytesLocalHeap(5, MemoryUnit.MEGABYTES)
+            .cache(configuration1)
+            .cache(configuration2)
+            .cache(new CacheConfiguration("three", 0));
+        configuration1.setMaxBytesLocalHeap("20%");
+        configuration2.setMaxBytesLocalHeap("20%");
+
+        CacheManager cacheManager = new CacheManager(configuration);
+        assertThat(cacheManager.getCache("one").getCacheConfiguration().getMaxBytesLocalHeap(), equalTo(MemoryUnit.MEGABYTES.toBytes(1)));
+        assertThat(cacheManager.getCache("two").getCacheConfiguration().getMaxBytesLocalHeap(), equalTo(MemoryUnit.MEGABYTES.toBytes(1)));
+        assertThat(cacheManager.getCache("three").getCacheConfiguration().getMaxBytesLocalHeap(), equalTo(0L));
+
+        configuration1 = new CacheConfiguration("one", 0);
+        configuration2 = new CacheConfiguration("two", 0);
+        configuration = new Configuration()
+            .cache(configuration1)
+            .cache(configuration2);
+        configuration1.setMaxBytesLocalHeap("2048");
+        configuration2.setMaxBytesLocalHeap("20%");
+        cacheManager.shutdown();
+
+        try {
+            new CacheManager(configuration);
+            fail("This should have thrown an InvalidConfigurationException");
+        } catch (InvalidConfigurationException e) {
+            assertThat(e.getMessage().contains("percentage maxBytesOnHeap"), is(true));
+            assertThat(e.getMessage().contains("no CacheManager wide value"), is(true));
+            assertThat(e.getMessage().contains("two"), is(true));
+        }
+    }
+
+    @Test
+    public void testDynamicallyAddedCacheConfiguration() {
+        Configuration configuration = new Configuration();
+        configuration.addCache(new CacheConfiguration("before1",100));
+        configuration.addCache(new CacheConfiguration("before2",100));
+        CacheManager manager = new CacheManager(configuration);
+        assertThat(manager.getCache("before1"), notNullValue());
+        assertThat(manager.getCache("before2"), notNullValue());
+        try {
+            configuration.addCache(new CacheConfiguration("after", 100));
+            fail("Should have had a IllegalStateException");
+        } catch (IllegalStateException e) {
+            assertThat(manager.getCache("after"), nullValue());
+        }
+        manager.shutdown();
+    }
+
+    @Test
+    public void testSupportsNameChanges() {
+        Configuration configuration = new Configuration().name("firstName");
+        CacheManager manager = new CacheManager(configuration);
+        assertThat(manager.getName(), equalTo("firstName"));
+
+        manager.setName("newerName");
+        assertThat(configuration.getName(), equalTo("newerName"));
+        assertThat(manager.getName(), equalTo("newerName"));
+
+        configuration.setName("evenNewerName");
+        assertThat(configuration.getName(), equalTo("evenNewerName"));
+        assertThat(manager.getName(), equalTo("evenNewerName"));
+        manager.shutdown();
+    }
+
+    @Test
+    public void testMaxBytesOverAllocated() {
+
+        Configuration configuration = new Configuration()
+            .maxBytesLocalHeap(50, MemoryUnit.KILOBYTES)
+            .cache(new CacheConfiguration("one", 0).maxBytesLocalHeap(30, MemoryUnit.KILOBYTES))
+            .cache(new CacheConfiguration("two", 0).maxBytesLocalHeap(30, MemoryUnit.KILOBYTES));
+
+        try {
+            new CacheManager(configuration);
+            fail("This should have thrown an InvalidConfigurationException");
+        } catch (InvalidConfigurationException e) {
+            assertThat(e.getMessage().contains("over-allocate"), is(true));
+        }
+
+        CacheConfiguration configuration1 = new CacheConfiguration("one", 0);
+        CacheConfiguration configuration2 = new CacheConfiguration("two", 0);
+        CacheConfiguration configuration3 = new CacheConfiguration("three", 0);
+        CacheConfiguration configuration4 = new CacheConfiguration("four", 0);
+        configuration = new Configuration()
+            .maxBytesLocalHeap(30, MemoryUnit.KILOBYTES)
+            .cache(configuration1)
+            .cache(configuration2)
+            .cache(configuration3)
+            .cache(configuration4);
+
+        configuration1.setMaxBytesLocalHeap("30%");
+        configuration2.setMaxBytesLocalHeap("30%");
+        configuration3.setMaxBytesLocalHeap("30%");
+        configuration4.setMaxBytesLocalHeap("30%");
+
+        try {
+            new CacheManager(configuration);
+            fail("This should have thrown an InvalidConfigurationException");
+        } catch (InvalidConfigurationException e) {
+            assertThat(e.getMessage().contains("over-allocate"), is(true));
+        }
+
+        configuration = new Configuration()
+            .maxBytesLocalHeap(4096, MemoryUnit.GIGABYTES)
+            .cache(new CacheConfiguration("one", 0).maxBytesLocalHeap(30, MemoryUnit.KILOBYTES))
+            .cache(new CacheConfiguration("two", 0).maxBytesLocalHeap(30, MemoryUnit.KILOBYTES));
+
+        try {
+            new CacheManager(configuration);
+            fail("This should have thrown an InvalidConfigurationException");
+        } catch (InvalidConfigurationException e) {
+            assertThat(e.getMessage().contains("-Xmx"), is(true));
+        }
+
+        configuration = new Configuration()
+            .cache(new CacheConfiguration("one", 0).maxBytesLocalHeap(2048, MemoryUnit.GIGABYTES))
+            .cache(new CacheConfiguration("two", 0).maxBytesLocalHeap(2048, MemoryUnit.GIGABYTES));
+
+        try {
+            new CacheManager(configuration);
+            fail("This should have thrown an InvalidConfigurationException");
+        } catch (InvalidConfigurationException e) {
+            assertThat(e.getMessage().contains("-Xmx"), is(true));
+        }
+
+        configuration1 = new CacheConfiguration("one", 0);
+        configuration2 = new CacheConfiguration("two", 0);
+        configuration3 = new CacheConfiguration("three", 0);
+        configuration4 = new CacheConfiguration("four", 0);
+        configuration = new Configuration()
+            .maxBytesLocalHeap(30, MemoryUnit.KILOBYTES)
+            .cache(configuration1)
+            .cache(configuration2)
+            .cache(configuration3);
+
+        configuration1.setMaxBytesLocalHeap("30%");
+        configuration2.setMaxBytesLocalHeap("30%");
+        configuration3.setMaxBytesLocalHeap("30%");
+        configuration4.setMaxBytesLocalHeap("30%");
+
+        CacheManager cacheManager = new CacheManager(configuration);
+        try {
+            cacheManager.addCache(new Cache(configuration4));
+            fail("This should have thrown an InvalidConfigurationException");
+        } catch (InvalidConfigurationException e) {
+            assertThat(e.getMessage().contains("over-allocate"), is(true));
+            assertThat(e.getMessage().contains("'four'"), is(true));
+        }
+        cacheManager.shutdown();
+    }
+
+    @Test
+    public void testPoolSize() throws Exception {
+        Configuration configuration = new Configuration()
+            .diskStore(new DiskStoreConfiguration().path("./tmp"))
+            .maxBytesLocalHeap(50, MemoryUnit.MEGABYTES)
+            .maxBytesLocalDisk(500, MemoryUnit.MEGABYTES)
+            .cache(new CacheConfiguration("one", 0).maxBytesLocalHeap(10, MemoryUnit.MEGABYTES))
+            .cache(new CacheConfiguration("two", 0).maxBytesLocalHeap(10, MemoryUnit.MEGABYTES))
+            .cache(new CacheConfiguration("three", 0).maxBytesLocalDisk(100, MemoryUnit.MEGABYTES));
+
+        CacheManager cacheManager = new CacheManager(configuration);
+        assertEquals(MemorySizeParser.parse("30M"), cacheManager.getOnHeapPool().getMaxSize());
+        assertEquals(MemorySizeParser.parse("400M"), cacheManager.getOnDiskPool().getMaxSize());
+
+        cacheManager.addCache(new Cache(new CacheConfiguration("four", 0)
+                .maxBytesLocalHeap(10, MemoryUnit.MEGABYTES)
+                .maxBytesLocalDisk(150, MemoryUnit.MEGABYTES)));
+        assertEquals(MemorySizeParser.parse("20M"), cacheManager.getOnHeapPool().getMaxSize());
+        assertEquals(MemorySizeParser.parse("250M"), cacheManager.getOnDiskPool().getMaxSize());
+
+        cacheManager.removeCache("one");
+        assertEquals(MemorySizeParser.parse("30M"), cacheManager.getOnHeapPool().getMaxSize());
+        assertEquals(MemorySizeParser.parse("250M"), cacheManager.getOnDiskPool().getMaxSize());
+
+        cacheManager.removeCache("three");
+        assertEquals(MemorySizeParser.parse("30M"), cacheManager.getOnHeapPool().getMaxSize());
+        assertEquals(MemorySizeParser.parse("350M"), cacheManager.getOnDiskPool().getMaxSize());
+    }
+
     @Test
     public void testCacheReferenceLookUps() {
         singletonManager = CacheManager.create();
         String cacheName = "randomNewCache";
-        singletonManager.addCache(cacheName);
+        singletonManager.addCache(new Cache(new CacheConfiguration().name(cacheName)));
 
         // Default state by name
         Cache cache = singletonManager.getCache(cacheName);
@@ -121,12 +428,14 @@ public class CacheManagerTest {
     public void testProgrammaticConfigurationFailsProperlyWhenNoDefaultCacheConfigured() {
         Configuration mgrConfig = new Configuration();
         mgrConfig.setUpdateCheck(false);
+        CacheManager cacheManager = null;
         try {
-            CacheManager cacheManager = new CacheManager(mgrConfig);
-            fail("This should have thrown an Exception, as no default cache is configured!");
+            cacheManager = new CacheManager(mgrConfig);
+            Assert.assertNotNull(cacheManager);
         } catch (Exception e) {
-            assertEquals("Illegal configuration. No default cache is configured.", e.getMessage());
+            fail("Creating cache manager having no default cache config shouldn't fail!");
         }
+        cacheManager.shutdown();
     }
 
     /**
@@ -137,7 +446,7 @@ public class CacheManagerTest {
         singletonManager = CacheManager.create();
         singletonManager.getEhcache("");
         assertNotNull(singletonManager);
-        assertEquals(14, singletonManager.getCacheNames().length);
+        assertEquals(CACHES_IN_EHCACHE_XML, singletonManager.getCacheNames().length);
     }
 
     /**
@@ -186,10 +495,11 @@ public class CacheManagerTest {
      */
     @Test
     public void testCreateTwoCacheManagersWithSamePath() throws CacheException {
-        URL secondCacheConfiguration = this.getClass().getResource(
+        URL configUrl = this.getClass().getResource(
                 "/ehcache-2.xml");
 
-        singletonManager = CacheManager.create(secondCacheConfiguration);
+        singletonManager = CacheManager.create(configUrl);
+        Configuration secondCacheConfiguration = ConfigurationFactory.parseConfiguration(configUrl).name("some-name");
         instanceManager = new CacheManager(secondCacheConfiguration);
 
         String intialDiskStorePath = System.getProperty("java.io.tmpdir")
@@ -211,6 +521,7 @@ public class CacheManagerTest {
         }
         assertTrue(newDiskStorePathFound);
         newDiskStorePath.delete();
+        instanceManager.shutdown();
 
     }
 
@@ -225,8 +536,9 @@ public class CacheManagerTest {
         CacheManager.getInstance().getCache("sampleCache1").put(element1);
 
         // Check can start second one with a different disk path
-        URL secondCacheConfiguration = this.getClass().getResource(
+        URL configUrl = this.getClass().getResource(
                 "/ehcache-2.xml");
+        Configuration secondCacheConfiguration = ConfigurationFactory.parseConfiguration(configUrl).name("cm-2");
         instanceManager = new CacheManager(secondCacheConfiguration);
         instanceManager.getCache("sampleCache1").put(element2);
 
@@ -253,6 +565,9 @@ public class CacheManagerTest {
                 1 + ""));
         assertEquals(element2, CacheManager.getInstance().getCache(
                 "sampleCache1").get(2 + ""));
+
+        CacheManager.getInstance().shutdown();
+        instanceManager.shutdown();
     }
 
     /**
@@ -267,8 +582,9 @@ public class CacheManagerTest {
         String fileName = AbstractCacheTest.TEST_CONFIG_DIR + "ehcache.xml";
         CacheManager.create(fileName).getCache("sampleCache1").put(element1);
 
+        Configuration secondConfig = ConfigurationFactory.parseConfiguration(new File(fileName)).name("cm-2");
         // Check can start second one with the same config
-        instanceManager = new CacheManager(fileName);
+        instanceManager = new CacheManager(secondConfig);
         instanceManager.getCache("sampleCache1").put(element2);
 
         assertEquals(element1, CacheManager.getInstance().getCache(
@@ -282,7 +598,7 @@ public class CacheManagerTest {
                 "sampleCache1").get(1 + ""));
 
         // Try shutting and recreating a new instance cache manager
-        instanceManager = new CacheManager(fileName);
+        instanceManager = new CacheManager(secondConfig);
         instanceManager.getCache("sampleCache1").put(element2);
         CacheManager.getInstance().shutdown();
         assertEquals(element2, instanceManager.getCache("sampleCache1").get(
@@ -294,6 +610,9 @@ public class CacheManagerTest {
                 1 + ""));
         assertEquals(element2, CacheManager.getInstance().getCache(
                 "sampleCache1").get(2 + ""));
+
+        CacheManager.getInstance().shutdown();
+        instanceManager.shutdown();
     }
 
     /**
@@ -336,7 +655,7 @@ public class CacheManagerTest {
      * <p/>
      * ehcache-1.2.3 had 126 threads for this test. ehcache-1.2.4 has 71. 70 for
      * the DiskStore thread and one shutdown hook
-     * <p />
+     * <p/>
      * ehcache-1.7 has 1 additional thread per cache for
      * SampledCacheUsageStatistics. 70 Caches means 140 threads plus 1 for
      * shutdown totalling to 141. Plus Junit thread totals 142.
@@ -359,18 +678,19 @@ public class CacheManagerTest {
     public void testInstanceCreateShutdownCreate() throws CacheException {
         singletonManager = CacheManager.create();
 
-        URL secondCacheConfiguration = this.getClass().getResource(
+        URL configUrl = this.getClass().getResource(
                 "/ehcache-2.xml");
+        Configuration secondCacheConfiguration = ConfigurationFactory.parseConfiguration(configUrl).name("cm-2");
         instanceManager = new CacheManager(secondCacheConfiguration);
         instanceManager.shutdown();
 
         // shutting down instance should leave singleton ok
-        assertEquals(14, singletonManager.getCacheNames().length);
+        assertEquals(CACHES_IN_EHCACHE_XML, singletonManager.getCacheNames().length);
 
         instanceManager = new CacheManager(secondCacheConfiguration);
         assertNotNull(instanceManager);
         assertEquals(8, instanceManager.getCacheNames().length);
-
+        instanceManager.shutdown();
     }
 
     /**
@@ -389,16 +709,11 @@ public class CacheManagerTest {
     @Test
     public void testCreateCacheManagersProgrammatically() throws CacheException {
 
-        Configuration configuration = new Configuration();
+        Configuration configuration = new Configuration()
+                .defaultCache(new CacheConfiguration("defaultCache", 10))
+                .diskStore(new DiskStoreConfiguration().path("java.io.tmpdir"));
         assertNotNull(configuration);
 
-        CacheConfiguration defaultCacheConfiguration = new CacheConfiguration("defaultCache", 10);
-        configuration.addDefaultCache(defaultCacheConfiguration);
-
-        DiskStoreConfiguration diskStoreConfiguration = new DiskStoreConfiguration();
-        diskStoreConfiguration.setPath("java.io.tmpdir");
-        configuration.addDiskStore(diskStoreConfiguration);
-
         instanceManager = new CacheManager(configuration);
         assertNotNull(instanceManager);
         assertEquals(0, instanceManager.getCacheNames().length);
@@ -500,7 +815,7 @@ public class CacheManagerTest {
         // check we can recreate the CacheManager on demand.
         singletonManager = CacheManager.create();
         assertNotNull(singletonManager);
-        assertEquals(14, singletonManager.getCacheNames().length);
+        assertEquals(CACHES_IN_EHCACHE_XML, singletonManager.getCacheNames().length);
         assertEquals(Status.STATUS_ALIVE, singletonManager.getStatus());
 
         singletonManager.shutdown();
@@ -513,15 +828,31 @@ public class CacheManagerTest {
     @Test
     public void testRemoveCache() throws CacheException {
         singletonManager = CacheManager.create();
+        assertEquals(15, singletonManager.getConfiguration().getCacheConfigurations().size());
         Ehcache cache = singletonManager.getCache("sampleCache1");
         assertNotNull(cache);
         singletonManager.removeCache("sampleCache1");
         cache = singletonManager.getCache("sampleCache1");
         assertNull(cache);
 
+        assertEquals(14, singletonManager.getConfiguration().getCacheConfigurations().size());
+
         // NPE tests
         singletonManager.removeCache(null);
         singletonManager.removeCache("");
+        assertEquals(14, singletonManager.getConfiguration().getCacheConfigurations().size());
+    }
+
+    @Test
+    public void testAddRemoveCache() throws CacheException {
+        String config = "<ehcache><defaultCache maxEntriesLocalHeap=\"0\"/></ehcache>";
+        CacheManager manager = new CacheManager(new ByteArrayInputStream(config.getBytes()));
+        assertEquals(0, manager.getConfiguration().getCacheConfigurations().size());
+        manager.addCache("test1");
+        assertEquals(1, manager.getConfiguration().getCacheConfigurations().size());
+        manager.removalAll();
+        assertEquals(0, manager.getConfiguration().getCacheConfigurations().size());
+        manager.shutdown();
     }
 
     /**
@@ -530,8 +861,10 @@ public class CacheManagerTest {
     @Test
     public void testAddCache() throws CacheException {
         singletonManager = CacheManager.create();
+        assertEquals(15, singletonManager.getConfiguration().getCacheConfigurations().size());
         singletonManager.addCache("test");
         singletonManager.addCache("test2");
+        assertEquals(17, singletonManager.getConfiguration().getCacheConfigurations().size());
         Ehcache cache = singletonManager.getCache("test");
         assertNotNull(cache);
         assertEquals("test", cache.getName());
@@ -546,6 +879,7 @@ public class CacheManagerTest {
 
         // NPE tests
         singletonManager.addCache("");
+        assertEquals(17, singletonManager.getConfiguration().getCacheConfigurations().size());
     }
 
     @Test
@@ -553,7 +887,7 @@ public class CacheManagerTest {
         singletonManager = CacheManager.create();
         singletonManager.addCache("present");
         assertTrue(singletonManager.getCache("present")
-                   == singletonManager.addCacheIfAbsent(new Cache(new CacheConfiguration("present", 1000))));
+                == singletonManager.addCacheIfAbsent(new Cache(new CacheConfiguration("present", 1000))));
 
         Cache theCache = new Cache(new CacheConfiguration("absent", 1000));
         Ehcache cache = singletonManager.addCacheIfAbsent(theCache);
@@ -575,7 +909,7 @@ public class CacheManagerTest {
         assertTrue(newCache == actualCacheRegisteredWithManager);
 
         assertTrue(singletonManager.addCacheIfAbsent(new Cache(new CacheConfiguration(actualCacheRegisteredWithManager.getName(), 1000)))
-                   == actualCacheRegisteredWithManager);
+                == actualCacheRegisteredWithManager);
 
         assertNull(singletonManager.addCacheIfAbsent((Ehcache) null));
     }
@@ -598,7 +932,7 @@ public class CacheManagerTest {
         assertTrue(singletonManager.addCacheIfAbsent(actualCacheRegisteredWithManager.getName()) == actualCacheRegisteredWithManager);
 
         assertTrue(singletonManager.addCacheIfAbsent(new Cache(new CacheConfiguration(actualCacheRegisteredWithManager.getName(), 1000)))
-                   == actualCacheRegisteredWithManager);
+                == actualCacheRegisteredWithManager);
         assertNull(singletonManager.addCacheIfAbsent((String) null));
         assertNull(singletonManager.addCacheIfAbsent(""));
     }
@@ -725,50 +1059,10 @@ public class CacheManagerTest {
             cache.get("key1");
             fail();
         } catch (IllegalStateException e) {
-            assertEquals("The test Cache is not alive.", e.getMessage());
+            assertEquals("The test Cache is not alive (STATUS_SHUTDOWN)", e.getMessage());
         }
     }
 
-    /**
-     * Tests that we can run 69 caches, most with disk stores, with no ill
-     * effects
-     * <p/>
-     * Check that this is fast.
-     */
-    @Test
-    public void testCreateCacheManagerWithManyCaches() throws CacheException,
-            InterruptedException {
-        singletonManager = CacheManager
-                .create(AbstractCacheTest.TEST_CONFIG_DIR + "ehcache-big.xml");
-        assertNotNull(singletonManager);
-        assertEquals(69, singletonManager.getCacheNames().length);
-
-        String[] names = singletonManager.getCacheNames();
-        for (int i = 0; i < names.length; i++) {
-            String name = names[i];
-            Ehcache cache = singletonManager.getCache(name);
-            for (int j = 0; i < 100; i++) {
-                cache.put(new Element(Integer.valueOf(j), "value"));
-            }
-        }
-        StopWatch stopWatch = new StopWatch();
-        for (int repeats = 0; repeats < 5000; repeats++) {
-            for (int i = 0; i < names.length; i++) {
-                String name = names[i];
-                Ehcache cache = singletonManager.getCache(name);
-                for (int j = 0; i < 100; i++) {
-                    Element element = cache.get(name + j);
-                    if ((element == null)) {
-                        cache.put(new Element(Integer.valueOf(j), "value"));
-                    }
-                }
-            }
-        }
-        long elapsedTime = stopWatch.getElapsedTime();
-        LOG.info("Time taken was: " + elapsedTime);
-        assertTrue("Time taken was: " + elapsedTime, elapsedTime < 5000);
-    }
-
     private int countThreads() {
         return JVMUtil.enumerateThreads().size();
     }
@@ -881,18 +1175,18 @@ public class CacheManagerTest {
     @Test
     public void testMultipleCacheManagers() {
         CacheManager[] managers = new CacheManager[2];
-        managers[0] = new CacheManager(makeCacheManagerConfig());
-        managers[1] = new CacheManager(makeCacheManagerConfig());
+        managers[0] = new CacheManager(makeCacheManagerConfig("cm1"));
+        managers[1] = new CacheManager(makeCacheManagerConfig("cm2"));
 
         managers[0].shutdown();
         managers[1].shutdown();
 
     }
 
-    private static Configuration makeCacheManagerConfig() {
-        Configuration config = new Configuration();
+    private static Configuration makeCacheManagerConfig(String cmName) {
+        Configuration config = new Configuration().name(cmName);
         CacheConfiguration defaults = new CacheConfiguration("cacheName", 10)
-            .eternal(true);
+                .eternal(true);
         config.setDefaultCacheConfiguration(defaults);
         return config;
     }
@@ -925,17 +1219,18 @@ public class CacheManagerTest {
         singletonManager.addCache(cacheB);
         Cache cacheC = new Cache("2", 10, false, false, 2, 2, false, 100);
         singletonManager.addCache(cacheC);
-        for(int i = 0; i < 100; i++) {
+        for (int i = 0; i < 100; i++) {
             cacheA.put(new Element(i + "", "dog"));
             cacheB.put(new Element(i + "", "dog"));
             cacheC.put(new Element(i + "", "dog"));
         }
         Cache diskCache = new Cache("disk", 10, true, false, 2, 2);
         try {
-          singletonManager.addCache(diskCache);
-          throw new AssertionError("Expected that adding a disk cache to a cache manager with no configured disk store path would throw CacheException");
+            singletonManager.addCache(diskCache);
+            throw new AssertionError("Expected that adding a disk cache to a cache manager" +
+                    " with no configured disk store path would throw CacheException");
         } catch (CacheException e) {
-          LOG.info("Caught expected exception", e);
+            LOG.info("Caught expected exception", e);
         }
         singletonManager.shutdown();
         assertEquals(null, singletonManager.getDiskStorePath());
@@ -966,7 +1261,7 @@ public class CacheManagerTest {
     public void testClearCacheManager() throws CacheException {
         singletonManager = CacheManager.create();
         assertNotNull(singletonManager);
-        assertEquals(14, singletonManager.getCacheNames().length);
+        assertEquals(CACHES_IN_EHCACHE_XML, singletonManager.getCacheNames().length);
         singletonManager.getEhcache("sampleCache1").put(new Element("key1", "value"));
         assertEquals(1, singletonManager.getEhcache("sampleCache1").getSize());
         singletonManager.getEhcache("sampleCache2").put(new Element("key2", "value"));
