@@ -16,11 +16,15 @@
 
 package net.sf.ehcache;
 
+import junit.framework.Assert;
+
+import net.sf.ehcache.pool.sizeof.JvmInformation;
 import net.sf.ehcache.store.LruMemoryStoreTest;
 import net.sf.ehcache.store.MemoryStore;
 import net.sf.ehcache.store.MemoryStoreEvictionPolicy;
 import net.sf.ehcache.store.Store;
 import org.junit.After;
+import org.junit.Assume;
 import org.junit.Before;
 import org.junit.Ignore;
 import org.junit.Test;
@@ -33,13 +37,15 @@ import java.util.Date;
 import java.util.List;
 import java.util.Random;
 
+import net.sf.ehcache.config.CacheConfiguration;
+import static org.hamcrest.Matchers.is;
 import static org.junit.Assert.*;
 
 /**
  * Other than policy differences, the Store implementations should work identically
  *
  * @author Greg Luck
- * @version $Id: MemoryStoreTester.java 2180 2010-04-07 21:30:58Z cdennis $
+ * @version $Id: MemoryStoreTester.java 4842 2011-10-13 13:42:15Z cdennis $
  */
 @Ignore
 public class MemoryStoreTester extends AbstractCacheTest {
@@ -108,7 +114,7 @@ public class MemoryStoreTester extends AbstractCacheTest {
     protected void createMemoryOnlyStore(MemoryStoreEvictionPolicy evictionPolicy) throws CacheException {
         manager.removeCache("testMemoryOnly");
         cache = new Cache("testMemoryOnly", 12000, evictionPolicy, false, System.getProperty("java.io.tmpdir"),
-                true, 60, 30, false, 60, null);
+                false, 60, 30, false, 60, null);
         manager.addCache(cache);
         store = cache.getStore();
     }
@@ -121,7 +127,7 @@ public class MemoryStoreTester extends AbstractCacheTest {
      */
     protected void createMemoryOnlyStore(MemoryStoreEvictionPolicy evictionPolicy, int memoryStoreSize) throws CacheException {
         manager.removeCache("test");
-        cache = new Cache("test", memoryStoreSize, evictionPolicy, false, null, true, 60, 30, false, 60, null);
+        cache = new Cache("test", memoryStoreSize, evictionPolicy, false, null, false, 60, 30, false, 60, null);
         manager.addCache(cache);
         store = cache.getStore();
     }
@@ -326,152 +332,6 @@ public class MemoryStoreTester extends AbstractCacheTest {
         assertNull(element);
     }
 
-    /**
-     * Tests bulk load.
-     */
-    @Test
-    public void testBulkLoad() throws Exception {
-        final Random random = new Random();
-        StopWatch stopWatch = new StopWatch();
-
-        // Add a bunch of entries
-        for (int i = 0; i < 500; i++) {
-            // Use a random length value
-            final String key = "key" + i;
-            final String value = "value" + random.nextInt(1000);
-
-            // Add an element, and make sure it is present
-            Element element = new Element(key, value);
-            store.put(element);
-            element = store.get(key);
-            assertNotNull(element);
-
-            // Remove the element
-            store.remove(key);
-            element = store.get(key);
-            assertNull(element);
-
-            element = new Element(key, value);
-            store.put(element);
-            element = store.get(key);
-            assertNotNull(element);
-        }
-        long time = stopWatch.getElapsedTime();
-        LOG.info("Time for Bulk Load: " + time);
-    }
-
-    /**
-     * Benchmark to test speed.
-     */
-    @Test
-    public void testBenchmarkPutGetRemove() throws Exception {
-        final String key = "key";
-        byte[] value = new byte[500];
-        StopWatch stopWatch = new StopWatch();
-
-        // Add a bunch of entries
-        for (int i = 0; i < 50000; i++) {
-            Element element = new Element(key, value);
-            store.put(element);
-            store.get(key + i);
-        }
-        for (int i = 0; i < 50000; i++) {
-            store.remove(key + i);
-        }
-        long time = stopWatch.getElapsedTime();
-        LOG.info("Time for benchmarkPutGetRemove: " + time);
-        assertTrue("Too slow. Time was " + time, time < 500);
-    }
-
-
-    /**
-     * Benchmark to test speed.
-     */
-    @Test
-    public void testBenchmarkPutGet() throws Exception {
-        final String key = "key";
-        byte[] value = new byte[500];
-        StopWatch stopWatch = new StopWatch();
-
-        // Add a bunch of entries
-        for (int i = 0; i < 50000; i++) {
-            Element element = new Element(key, value);
-            store.put(element);
-        }
-        for (int i = 0; i < 50000; i++) {
-            store.get(key + i);
-        }
-        long time = stopWatch.getElapsedTime();
-        LOG.info("Time for benchmarkPutGet: " + time);
-        assertTrue("Too slow. Time was " + time, time < 300);
-    }
-
-
-    /**
-     * Benchmark to test speed.
-     * <p/>
-     * With iteration up to 5000:
-     * 100: Time for putSpeed: 2772
-     * 1000: Time for putSpeed: 10943
-     * 4000: Time for putSpeed: 42367
-     * 10000: Time for putSpeed: 4179
-     * 300000: Time for putSpeed: 6616
-     * <p/>
-     * With no iteration:
-     * 100: Time for putSpeed: 2358
-     * 1000: Time for putSpeed: 2692
-     * 4000: Time for putSpeed: 2833
-     * 10000: Time for putSpeed: 3630
-     * 300000: Time for putSpeed: 6616
-     */
-    @Test
-    public void testPutSpeed() throws Exception {
-        cache = new Cache("testPutSpeed", 4000, false, true, 120, 120);
-        manager.addCache(cache);
-        store = cache.getStore();
-        final Long key = 0L;
-        byte[] value = new byte[1];
-        StopWatch stopWatch = new StopWatch();
-
-        // Add a bunch of entries
-        for (int i = 0; i < 500000; i++) {
-            Element element = new Element(key + i, value);
-            store.put(element);
-        }
-        long time = stopWatch.getElapsedTime();
-        LOG.info("Time for putSpeed: " + time);
-        assertTrue("Too slow. Time was " + time, time < 4000);
-    }
-
-
-    /**
-     * Benchmark to test speed.
-     * Original implementation 12seconds
-     * This implementation 9 seconds
-     */
-    public void benchmarkPutGetSuryaTest(long allowedTime) throws Exception {
-        Random random = new Random();
-        byte[] value = new byte[500];
-        StopWatch stopWatch = new StopWatch();
-
-        // Add a bunch of entries
-        for (int i = 0; i < 50000; i++) {
-            String key = "key" + i;
-
-            Element element = new Element(key, value);
-            store.put(element);
-
-            //Access each element random number of times, min:0 maximum:9
-            int accesses = random.nextInt(5);
-            for (int j = 0; j <= accesses; j++) {
-                store.get(key);
-            }
-        }
-        long time = stopWatch.getElapsedTime();
-        LOG.info("Time for benchmarkPutGetSurya: " + time);
-        assertTrue("Too slow. Time was " + time, time < allowedTime);
-    }
-
     /**
      * Multi-thread read-only test.
      */
@@ -610,6 +470,7 @@ public class MemoryStoreTester extends AbstractCacheTest {
      */
     @Test
     public void testReadWriteThreadsSurya() throws Exception {
+        Assume.assumeThat(JvmInformation.isJRockit(), is(false));
 
         long start = System.currentTimeMillis();
         final List executables = new ArrayList();
@@ -648,66 +509,37 @@ public class MemoryStoreTester extends AbstractCacheTest {
         runThreads(executables);
         long end = System.currentTimeMillis();
         LOG.info("Total time for the test: " + (end + start) + " ms");
+        tearDown();
+        System.gc();
+        System.gc();
+        Thread.sleep(1000);
+        System.gc();
+        System.gc();
     }
 
 
     /**
-     * Test behaviour of memory store using 1 million records.
-     * This is expected to run out of memory on a 64MB machine. Where it runs out
-     * is asserted so that design changes do not start using more memory per element.
-     * <p/>
-     * This test will fail (ie not get an out of memory error) on VMs configured to be server which do not have a fixed upper memory limit.
-     * <p/>
-     * Takes too long to run therefore switch off
-     * <p/>
-     * These memory size asserts were 100,000 and 60,000. The ApacheLRU map does not get quite as high numbers.
-     * This test varies according to architecture. 64 bit architectures
+     * Test that the memory store can fit 65k elements in a 64Mb heap.
      */
     @Test
     public void testMemoryStoreOutOfMemoryLimit() throws Exception {
+        Assume.assumeThat(JvmInformation.isJRockit(), is(false));
         LOG.info("Starting out of memory limit test");
         //Set size so the second element overflows to disk.
         cache = manager.getCache("memoryLimitTest");
         if (cache == null) {
-            cache = new Cache("memoryLimitTest", 1000000, false, false, 500, 500);
+            cache = new Cache(new CacheConfiguration("memoryLimitTest", 0));
             manager.addCache(cache);
         }
-        int i = 0;
-        try {
-            for (; i < 1000000; i++) {
-                cache.put(new Element("" +
-                        i, "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
-                        + "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
-                        + "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
-                        + "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
-                        + "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
-                        + "AAAAA " + i));
-            }
-            LOG.info("About to fail out of memory limit test");
-            fail();
-        } catch (OutOfMemoryError e) {
-            cache.removeAll();
-            Thread.sleep(1000);
-            System.gc();
-            Thread.sleep(2000);
-            System.gc();
-
-            try {
-                LOG.info("Ran out of memory putting " + i + "th element");
-                assertTrue(i > 65000);
-            } catch (OutOfMemoryError e1) {
-                //sometimes if we are really out of memory we cannot do anything
-            }
-
+        for (int i = 0; i < 65000; i++) {
+            cache.put(new Element(Integer.valueOf(i), new String(new char[218])));
         }
-        Thread.sleep(1000);
-        System.gc();
-        Thread.sleep(1000);
+        assertEquals(65000, cache.getSize());
     }
 
     @Test
     public void testShrinkingAndGrowingMemoryStore() {
-        cache = new Cache("testShrinkingAndGrowingMemoryStore", 50, false, true, 120, 120);
+        cache = new Cache("testShrinkingAndGrowingMemoryStore", 50, false, false, 120, 120);
         manager.addCache(cache);
         store = cache.getStore();
 
@@ -717,7 +549,7 @@ public class MemoryStoreTester extends AbstractCacheTest {
         }
 
         int i = 0;
-        for (;;) {
+        for (; ;) {
             int size = store.getSize();
             store.put(new Element(Integer.valueOf(i++), new byte[100]));
             if (store.getSize() <= size) break;
@@ -727,7 +559,7 @@ public class MemoryStoreTester extends AbstractCacheTest {
         final int shrinkSize = initialSize / 2;
         ((MemoryStore) store).memoryCapacityChanged(initialSize, shrinkSize);
 
-        for (;;) {
+        for (; ;) {
             int size = store.getSize();
             store.put(new Element(Integer.valueOf(i++), new byte[100]));
             if (store.getSize() >= size) break;
@@ -742,7 +574,7 @@ public class MemoryStoreTester extends AbstractCacheTest {
         final int growSize = initialSize * 2;
         ((MemoryStore) store).memoryCapacityChanged(shrinkSize, growSize);
 
-        for (;;) {
+        for (; ;) {
             int size = store.getSize();
             store.put(new Element(Integer.valueOf(i++), new byte[100]));
             if (store.getSize() <= size) break;
@@ -754,4 +586,39 @@ public class MemoryStoreTester extends AbstractCacheTest {
             assertTrue(size > (growSize * 0.9));
         }
     }
+
+    @Test
+    public void testElementPinning() throws Exception {
+        createMemoryOnlyStore(MemoryStoreEvictionPolicy.LRU, 20);
+
+        for (int i = 0; i < 200; i++) {
+            Element element = new Element("Ku-" + i, "@" + i);
+            store.put(element);
+        }
+
+        Assert.assertEquals(20, store.getSize());
+
+        for (int i = 0; i < 200; i++) {
+            Element element = new Element("Kp-" + i, "#" + i);
+            element.setTimeToIdle(1);
+            element.setTimeToLive(1);
+            store.setPinned(element.getObjectKey(), true);
+            store.put(element);
+        }
+
+        for (int i = 0; i < 200; i++) {
+            assertTrue("missing key Kp-" + i, store.containsKey("Kp-" + i));
+        }
+
+        // wait until all pinned elements expire
+        Thread.sleep(1100);
+
+        for (int i = 1000; i < 1200; i++) {
+            Element element = new Element("Ku-" + i, "#" + i);
+            store.put(element);
+        }
+
+        Assert.assertEquals(20, store.getSize());
+    }
+
 }
