@@ -17,32 +17,12 @@
 
 package net.sf.ehcache.config;
 
-import net.sf.ehcache.AbstractCacheTest;
-import net.sf.ehcache.Cache;
-import net.sf.ehcache.CacheException;
-import net.sf.ehcache.CacheManager;
-import net.sf.ehcache.Ehcache;
-import net.sf.ehcache.bootstrap.BootstrapCacheLoader;
-import net.sf.ehcache.distribution.CacheManagerPeerListener;
-import net.sf.ehcache.distribution.CacheManagerPeerProvider;
-import net.sf.ehcache.distribution.MulticastRMICacheManagerPeerProvider;
-import net.sf.ehcache.distribution.RMIAsynchronousCacheReplicator;
-import net.sf.ehcache.distribution.RMIBootstrapCacheLoader;
-import net.sf.ehcache.distribution.RMICacheManagerPeerListener;
-import net.sf.ehcache.distribution.RMICacheReplicatorFactory;
-import net.sf.ehcache.event.CacheEventListener;
-import net.sf.ehcache.event.CacheManagerEventListener;
-import net.sf.ehcache.event.CountingCacheEventListener;
-import net.sf.ehcache.event.CountingCacheManagerEventListener;
-import net.sf.ehcache.event.NotificationScope;
-import net.sf.ehcache.exceptionhandler.CacheExceptionHandler;
-import net.sf.ehcache.exceptionhandler.CountingExceptionHandler;
-import net.sf.ehcache.store.compound.SerializationCopyStrategy;
-import net.sf.ehcache.writer.TestCacheWriter;
-import org.junit.Before;
-import org.junit.Test;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNotSame;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
@@ -63,12 +43,37 @@ import java.util.jar.JarEntry;
 import java.util.jar.JarOutputStream;
 import java.util.regex.Matcher;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertNotSame;
-import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
+import junit.framework.Assert;
+import net.sf.ehcache.AbstractCacheTest;
+import net.sf.ehcache.Cache;
+import net.sf.ehcache.CacheException;
+import net.sf.ehcache.CacheManager;
+import net.sf.ehcache.Ehcache;
+import net.sf.ehcache.bootstrap.BootstrapCacheLoader;
+import net.sf.ehcache.config.TerracottaConfiguration.Consistency;
+import net.sf.ehcache.distribution.CacheManagerPeerListener;
+import net.sf.ehcache.distribution.CacheManagerPeerProvider;
+import net.sf.ehcache.distribution.MulticastRMICacheManagerPeerProvider;
+import net.sf.ehcache.distribution.RMIAsynchronousCacheReplicator;
+import net.sf.ehcache.distribution.RMIBootstrapCacheLoader;
+import net.sf.ehcache.distribution.RMICacheManagerPeerListener;
+import net.sf.ehcache.distribution.RMICacheReplicatorFactory;
+import net.sf.ehcache.event.CacheEventListener;
+import net.sf.ehcache.event.CacheManagerEventListener;
+import net.sf.ehcache.event.CountingCacheEventListener;
+import net.sf.ehcache.event.CountingCacheManagerEventListener;
+import net.sf.ehcache.event.NotificationScope;
+import net.sf.ehcache.exceptionhandler.CacheExceptionHandler;
+import net.sf.ehcache.exceptionhandler.CountingExceptionHandler;
+import net.sf.ehcache.store.DefaultElementValueComparator;
+import net.sf.ehcache.store.MemoryStoreEvictionPolicy;
+import net.sf.ehcache.store.compound.ReadWriteSerializationCopyStrategy;
+import net.sf.ehcache.writer.TestCacheWriter;
+
+import org.junit.Before;
+import org.junit.Test;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Tests for Store Configuration
@@ -77,10 +82,10 @@ import static org.junit.Assert.fail;
  * in the classpath.
  *
  * @author <a href="mailto:gluck@thoughtworks.com">Greg Luck</a>
- * @version $Id: ConfigurationFactoryTest.java 2199 2010-04-08 21:02:44Z asingh $
+ * @version $Id: ConfigurationFactoryTest.java 4748 2011-09-27 15:58:15Z lorban $
  */
 public class ConfigurationFactoryTest extends AbstractCacheTest {
-    private static final int CACHES_IN_TEST_EHCACHE = 14;
+    private static final int CACHES_IN_TEST_EHCACHE = 15;
 
     private static final Logger LOG = LoggerFactory.getLogger(ConfigurationFactoryTest.class.getName());
 
@@ -133,7 +138,7 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         assertEquals(Integer.valueOf(0), ((MulticastRMICacheManagerPeerProvider) peerProvider).getHeartBeatSender().getTimeToLive());
 
         //Check CacheManagerEventListener
-        assertEquals(null, configurationHelper.createCacheManagerEventListener());
+        assertEquals(null, configurationHelper.createCacheManagerEventListener(null));
 
         //Check default cache
         Ehcache defaultCache = configurationHelper.createDefaultCache();
@@ -359,7 +364,7 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         ConfigurationHelper configurationHelper = new ConfigurationHelper(manager, configuration);
 
         //Check CacheManagerEventListener
-        CacheManagerEventListener listener = configurationHelper.createCacheManagerEventListener();
+        CacheManagerEventListener listener = configurationHelper.createCacheManagerEventListener(null);
         assertEquals(null, listener);
 
         //Check caches. Configuration should have completed
@@ -379,7 +384,7 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         //Check CacheManagerEventListener
         CacheManagerEventListener listener = null;
         try {
-            listener = configurationHelper.createCacheManagerEventListener();
+            listener = configurationHelper.createCacheManagerEventListener(null);
             fail();
         } catch (CacheException e) {
             //expected
@@ -396,7 +401,7 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         ConfigurationHelper configurationHelper = new ConfigurationHelper(manager, configuration);
 
         //Check CacheManagerEventListener
-        Class actualClass = configurationHelper.createCacheManagerEventListener().getClass();
+        Class actualClass = configurationHelper.createCacheManagerEventListener(null).getClass();
         assertEquals(CountingCacheManagerEventListener.class, actualClass);
 
         //Check caches. Configuration should have completed
@@ -536,10 +541,10 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
 
         //Check default cache
         try {
-            configurationHelper.createDefaultCache();
-            fail();
+            Ehcache defaultCache = configurationHelper.createDefaultCache();
+            Assert.assertNull(defaultCache);
         } catch (CacheException e) {
-            //noop
+            fail("Calling create default cache when no default cache config specified should return null and not fail");
         }
 
         //Check caches
@@ -554,17 +559,18 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         //  />
         Ehcache sampleCache1 = configurationHelper.createCacheFromName("sampleCache1");
         Ehcache sampleCache4 = configurationHelper.createCacheFromName("sampleCache4");
-        assertEquals("net.sf.ehcache.transaction.manager.DefaultTransactionManagerLookup", configuration.getTransactionManagerLookupConfiguration().getFullyQualifiedClassPath());
+        assertEquals("net.sf.ehcache.transaction.manager.DefaultTransactionManagerLookup",
+                configuration.getTransactionManagerLookupConfiguration().getFullyQualifiedClassPath());
         assertEquals("sampleCache1", sampleCache1.getName());
         assertEquals(false, sampleCache1.getCacheConfiguration().isEternal());
         assertEquals(300, sampleCache1.getCacheConfiguration().getTimeToIdleSeconds());
         assertEquals(600, sampleCache1.getCacheConfiguration().getTimeToLiveSeconds());
         assertEquals(true, sampleCache1.getCacheConfiguration().isOverflowToDisk());
         assertEquals(CacheConfiguration.TransactionalMode.OFF, sampleCache1.getCacheConfiguration().getTransactionalMode());
-        assertEquals(false, sampleCache1.getCacheConfiguration().isTransactional());
+        assertEquals(false, sampleCache1.getCacheConfiguration().isXaStrictTransactional());
         assertEquals("sampleCache4", sampleCache4.getName());
-        assertEquals(CacheConfiguration.TransactionalMode.XA, sampleCache4.getCacheConfiguration().getTransactionalMode());
-        assertEquals(true, sampleCache4.getCacheConfiguration().isTransactional());
+        assertEquals(CacheConfiguration.TransactionalMode.XA_STRICT, sampleCache4.getCacheConfiguration().getTransactionalMode());
+        assertEquals(true, sampleCache4.getCacheConfiguration().isXaStrictTransactional());
     }
 
     /**
@@ -695,9 +701,12 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
      */
     @Test
     public void testEmptyPeerListManualDistributedConfiguration() {
-        CacheManager cacheManager = new CacheManager(TEST_CONFIG_DIR + "distribution/ehcache-manual-distributed3.xml");
+        Configuration config = ConfigurationFactory.parseConfiguration(
+                new File(TEST_CONFIG_DIR + "distribution/ehcache-manual-distributed3.xml")).name("new-name");
+        CacheManager cacheManager = new CacheManager(config);
         assertEquals(0, cacheManager.getCacheManagerPeerProvider("RMI")
                 .listRemoteCachePeers(cacheManager.getCache("sampleCache1")).size());
+        cacheManager.shutdown();
 
     }
 
@@ -973,6 +982,7 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
     @Test
     public void testLoadConfigurationWithReplacement() throws Exception {
         System.setProperty("multicastGroupPort", "4446");
+        System.setProperty("serverAndPort", "server.com:9510");
         File file = new File(TEST_CONFIG_DIR + "ehcache-replacement.xml");
         Configuration configuration = ConfigurationFactory.parseConfiguration(file);
         ConfigurationHelper configurationHelper = new ConfigurationHelper(manager, configuration);
@@ -1022,6 +1032,19 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         assertEquals("${multicastAddress}", firstPropertyToken);
     }
 
+    /**
+     * Tests the property token extraction logic
+     */
+    @Test
+    public void testMatchPropertyTokensProperlyFormedUrl() {
+        String example = "<terracottaConfig url=\"${serverAndPort}\"/>";
+        Set propertyTokens = ConfigurationFactory.extractPropertyTokens(example);
+        assertEquals(1, propertyTokens.size());
+        String firstPropertyToken = (String) (propertyTokens.toArray())[0];
+        assertEquals("${serverAndPort}", firstPropertyToken);
+    }
+
+
     /**
      * Tests the property token extraction logic
      */
@@ -1074,13 +1097,13 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         assertTrue(copyOnReadCache.getCacheConfiguration().isCopyOnRead());
         assertFalse(copyOnReadCache.getCacheConfiguration().isCopyOnWrite());
         assertNotNull(copyOnReadCache.getCacheConfiguration().getCopyStrategy());
-        assertTrue(copyOnReadCache.getCacheConfiguration().getCopyStrategy() instanceof SerializationCopyStrategy);
+        assertTrue(copyOnReadCache.getCacheConfiguration().getCopyStrategy() instanceof ReadWriteSerializationCopyStrategy);
 
         Ehcache copyOnWriteCache = configurationHelper.createCacheFromName("copyOnWriteCache");
         assertFalse(copyOnWriteCache.getCacheConfiguration().isCopyOnRead());
         assertTrue(copyOnWriteCache.getCacheConfiguration().isCopyOnWrite());
         assertNotNull(copyOnWriteCache.getCacheConfiguration().getCopyStrategy());
-        assertTrue(copyOnWriteCache.getCacheConfiguration().getCopyStrategy() instanceof SerializationCopyStrategy);
+        assertTrue(copyOnWriteCache.getCacheConfiguration().getCopyStrategy() instanceof ReadWriteSerializationCopyStrategy);
 
         Ehcache copyCache = configurationHelper.createCacheFromName("copyCache");
         assertTrue(copyCache.getCacheConfiguration().isCopyOnRead());
@@ -1089,17 +1112,18 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         assertTrue(copyCache.getCacheConfiguration().getCopyStrategy() instanceof FakeCopyStrategy);
 
         try {
-            new CacheManager(TEST_CONFIG_DIR + "ehcache-copy.xml");
+            Configuration config = ConfigurationFactory.parseConfiguration(new File(TEST_CONFIG_DIR + "ehcache-copy.xml")).name("new-cm");
+            new CacheManager(config);
             fail("This should have thrown an Exception");
         } catch (Exception e) {
-            if(!(e instanceof InvalidConfigurationException)) {
+            if (!(e instanceof InvalidConfigurationException)) {
                 e.printStackTrace();
-                fail("Expected InvalidConfigurationException, but got "+ e.getClass().getSimpleName());
+                fail("Expected InvalidConfigurationException, but got " + e.getClass().getSimpleName() + ", msg: " + e.getMessage());
             }
         }
-        
+
         file = new File(TEST_CONFIG_DIR + "ehcache-copy-tc.xml");
-        configuration = ConfigurationFactory.parseConfiguration(file);
+        configuration = ConfigurationFactory.parseConfiguration(file).name("new-cm");
         configurationHelper = new ConfigurationHelper(manager, configuration);
 
         Ehcache nonCopyCache = configurationHelper.createCacheFromName("nonCopyOnReadCacheTcTrue");
@@ -1115,6 +1139,21 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         assertTrue(copyOnReadCacheTc.getCacheConfiguration().getTerracottaConfiguration().isCopyOnRead());
     }
 
+    @Test
+    public void testElementValueComparatorConfiguration() {
+        File file = new File(TEST_CONFIG_DIR + "ehcache-comparator.xml");
+        Configuration configuration = ConfigurationFactory.parseConfiguration(file);
+        ConfigurationHelper configurationHelper = new ConfigurationHelper(manager, configuration);
+
+        Ehcache cache = configurationHelper.createCacheFromName("cache");
+        assertTrue(cache.getCacheConfiguration().getElementValueComparatorConfiguration().getElementComparatorInstance(cache.getCacheConfiguration())
+                instanceof DefaultElementValueComparator);
+
+        Ehcache cache2 = configurationHelper.createCacheFromName("cache2");
+        assertTrue(cache2.getCacheConfiguration().getElementValueComparatorConfiguration().getElementComparatorInstance(cache.getCacheConfiguration())
+                .getClass().equals(FakeElementValueComparator.class));
+    }
+
     /**
      * Test named cachemanager, terracotta config, clustered caches
      */
@@ -1241,53 +1280,54 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         assertEquals(true, sampleCache9.getCacheConfiguration().isTerracottaClustered());
         assertEquals(42,
                 sampleCache9.getCacheConfiguration().getTerracottaConfiguration().getLocalKeyCacheSize());
-      
-      // assert default value is true always
-      assertEquals(true, TerracottaConfiguration.DEFAULT_CACHE_COHERENT);
-      
-      Ehcache sampleCache10 = configurationHelper.createCacheFromName("clustered-10");
-      assertEquals("clustered-10", sampleCache10.getName());
-      assertEquals(true, sampleCache10.getCacheConfiguration().isTerracottaClustered());
-      assertEquals(true,
-              sampleCache10.getCacheConfiguration().getTerracottaConfiguration().isCoherent());
-      
-      Ehcache sampleCache11 = configurationHelper.createCacheFromName("clustered-11");
-      assertEquals("clustered-11", sampleCache11.getName());
-      assertEquals(true, sampleCache11.getCacheConfiguration().isTerracottaClustered());
-      assertEquals(false,
-              sampleCache11.getCacheConfiguration().getTerracottaConfiguration().isCoherent());
-      
-      Ehcache sampleCache12 = configurationHelper.createCacheFromName("clustered-12");
-      assertEquals("clustered-12", sampleCache12.getName());
-      assertEquals(true, sampleCache12.getCacheConfiguration().isTerracottaClustered());
-      assertEquals(true,
-              sampleCache12.getCacheConfiguration().getTerracottaConfiguration().isCoherent());
-      
-   // assert default value is false always
-      assertEquals(false, TerracottaConfiguration.DEFAULT_SYNCHRONOUS_WRITES);
-      
-      Ehcache sampleCache13 = configurationHelper.createCacheFromName("clustered-13");
-      assertEquals("clustered-13", sampleCache13.getName());
-      assertEquals(true, sampleCache13.getCacheConfiguration().isTerracottaClustered());
-      assertEquals(false,
-              sampleCache13.getCacheConfiguration().getTerracottaConfiguration().isSynchronousWrites());
-      
-      Ehcache sampleCache14 = configurationHelper.createCacheFromName("clustered-14");
-      assertEquals("clustered-14", sampleCache14.getName());
-      assertEquals(true, sampleCache14.getCacheConfiguration().isTerracottaClustered());
-      assertEquals(false,
-              sampleCache14.getCacheConfiguration().getTerracottaConfiguration().isSynchronousWrites());
-      
-      Ehcache sampleCache15 = configurationHelper.createCacheFromName("clustered-15");
-      assertEquals("clustered-15", sampleCache15.getName());
-      assertEquals(true, sampleCache15.getCacheConfiguration().isTerracottaClustered());
-      assertEquals(true,
-              sampleCache15.getCacheConfiguration().getTerracottaConfiguration().isSynchronousWrites());
+
+        // assert default value is true always
+        assertEquals(true, TerracottaConfiguration.DEFAULT_CACHE_COHERENT);
+
+        Ehcache sampleCache10 = configurationHelper.createCacheFromName("clustered-10");
+        assertEquals("clustered-10", sampleCache10.getName());
+        assertEquals(true, sampleCache10.getCacheConfiguration().isTerracottaClustered());
+        final boolean expectedDefault = TerracottaConfiguration.DEFAULT_CONSISTENCY_TYPE == Consistency.STRONG? true: false;
+        assertEquals(expectedDefault,
+                sampleCache10.getCacheConfiguration().getTerracottaConfiguration().isCoherent());
+
+        Ehcache sampleCache11 = configurationHelper.createCacheFromName("clustered-11");
+        assertEquals("clustered-11", sampleCache11.getName());
+        assertEquals(true, sampleCache11.getCacheConfiguration().isTerracottaClustered());
+        assertEquals(false,
+                sampleCache11.getCacheConfiguration().getTerracottaConfiguration().isCoherent());
+
+        Ehcache sampleCache12 = configurationHelper.createCacheFromName("clustered-12");
+        assertEquals("clustered-12", sampleCache12.getName());
+        assertEquals(true, sampleCache12.getCacheConfiguration().isTerracottaClustered());
+        assertEquals(true,
+                sampleCache12.getCacheConfiguration().getTerracottaConfiguration().isCoherent());
+
+        // assert default value is false always
+        assertEquals(false, TerracottaConfiguration.DEFAULT_SYNCHRONOUS_WRITES);
+
+        Ehcache sampleCache13 = configurationHelper.createCacheFromName("clustered-13");
+        assertEquals("clustered-13", sampleCache13.getName());
+        assertEquals(true, sampleCache13.getCacheConfiguration().isTerracottaClustered());
+        assertEquals(false,
+                sampleCache13.getCacheConfiguration().getTerracottaConfiguration().isSynchronousWrites());
+
+        Ehcache sampleCache14 = configurationHelper.createCacheFromName("clustered-14");
+        assertEquals("clustered-14", sampleCache14.getName());
+        assertEquals(true, sampleCache14.getCacheConfiguration().isTerracottaClustered());
+        assertEquals(false,
+                sampleCache14.getCacheConfiguration().getTerracottaConfiguration().isSynchronousWrites());
+
+        Ehcache sampleCache15 = configurationHelper.createCacheFromName("clustered-15");
+        assertEquals("clustered-15", sampleCache15.getName());
+        assertEquals(true, sampleCache15.getCacheConfiguration().isTerracottaClustered());
+        assertEquals(true,
+                sampleCache15.getCacheConfiguration().getTerracottaConfiguration().isSynchronousWrites());
 
         // <terracottaConfig>
         //  <url>localhost:9510</url>
         // </terracottaConfig>
-        TerracottaConfigConfiguration tcConfig = configuration.getTerracottaConfiguration();
+        TerracottaClientConfiguration tcConfig = configuration.getTerracottaConfiguration();
         assertNotNull(tcConfig);
         assertEquals("localhost:9510", tcConfig.getUrl());
     }
@@ -1334,7 +1374,7 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         // <terracottaConfig>
         //  <tc-config> ... </tc-config>
         // </terracottaConfig>
-        TerracottaConfigConfiguration tcConfig = configuration.getTerracottaConfiguration();
+        TerracottaClientConfiguration tcConfig = configuration.getTerracottaConfiguration();
         assertNotNull(tcConfig);
         assertEquals(null, tcConfig.getUrl());
         String embeddedConfig = tcConfig.getEmbeddedConfig();
@@ -1363,7 +1403,7 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         // <terracottaConfig>
         // <tc-config> ... </tc-config>
         // </terracottaConfig>
-        TerracottaConfigConfiguration tcConfig = configuration
+        TerracottaClientConfiguration tcConfig = configuration
                 .getTerracottaConfiguration();
         assertNotNull(tcConfig);
         assertEquals(null, tcConfig.getUrl());
@@ -1413,7 +1453,8 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         Configuration configuration = ConfigurationFactory.parseConfiguration(file);
         List configs = configuration.getCacheConfigurations().get("clustered").getCacheEventListenerConfigurations();
         assertEquals(1, configs.size());
-        assertEquals(((CacheConfiguration.CacheEventListenerFactoryConfiguration)configs.get(0)).getFullyQualifiedClassPath(), RMICacheReplicatorFactory.class.getName());
+        assertEquals(((CacheConfiguration.CacheEventListenerFactoryConfiguration) configs.get(0)).getFullyQualifiedClassPath(),
+                RMICacheReplicatorFactory.class.getName());
     }
 
     /**
@@ -1425,7 +1466,8 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         Configuration configuration = ConfigurationFactory.parseConfiguration(file);
         List configs = configuration.getCacheConfigurations().get("clustered").getCacheEventListenerConfigurations();
         assertEquals(1, configs.size());
-        assertEquals(((CacheConfiguration.CacheEventListenerFactoryConfiguration)configs.get(0)).getFullyQualifiedClassPath(), "net.sf.ehcache.distribution.JGroupsCacheReplicatorFactory");
+        assertEquals(((CacheConfiguration.CacheEventListenerFactoryConfiguration) configs.get(0)).getFullyQualifiedClassPath(),
+                "net.sf.ehcache.distribution.JGroupsCacheReplicatorFactory");
     }
 
     /**
@@ -1437,7 +1479,8 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         Configuration configuration = ConfigurationFactory.parseConfiguration(file);
         List configs = configuration.getCacheConfigurations().get("clustered").getCacheEventListenerConfigurations();
         assertEquals(1, configs.size());
-        assertEquals(((CacheConfiguration.CacheEventListenerFactoryConfiguration)configs.get(0)).getFullyQualifiedClassPath(), "net.sf.ehcache.distribution.JMSCacheReplicatorFactory");
+        assertEquals(((CacheConfiguration.CacheEventListenerFactoryConfiguration) configs.get(0)).getFullyQualifiedClassPath(),
+                "net.sf.ehcache.distribution.JMSCacheReplicatorFactory");
     }
 
     private String removeLotsOfWhitespace(String str) {
@@ -1490,15 +1533,18 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         assertEquals(10, configDefault.getMaxElementsInMemory());
         assertNotNull(configDefault.getCacheWriterConfiguration());
         assertEquals(defaultCacheWriterConfig.getWriteMode(), configDefault.getCacheWriterConfiguration().getWriteMode());
-        assertEquals(defaultCacheWriterConfig.getCacheWriterFactoryConfiguration(), configDefault.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration());
-        assertEquals(defaultCacheWriterConfig.getNotifyListenersOnException(), configDefault.getCacheWriterConfiguration().getNotifyListenersOnException());
+        assertEquals(defaultCacheWriterConfig.getCacheWriterFactoryConfiguration(), configDefault.getCacheWriterConfiguration()
+                .getCacheWriterFactoryConfiguration());
+        assertEquals(defaultCacheWriterConfig.getNotifyListenersOnException(), configDefault.getCacheWriterConfiguration()
+                .getNotifyListenersOnException());
         assertEquals(defaultCacheWriterConfig.getMaxWriteDelay(), configDefault.getCacheWriterConfiguration().getMaxWriteDelay());
         assertEquals(defaultCacheWriterConfig.getRateLimitPerSecond(), configDefault.getCacheWriterConfiguration().getRateLimitPerSecond());
         assertEquals(defaultCacheWriterConfig.getWriteCoalescing(), configDefault.getCacheWriterConfiguration().getWriteCoalescing());
         assertEquals(defaultCacheWriterConfig.getWriteBatching(), configDefault.getCacheWriterConfiguration().getWriteBatching());
         assertEquals(defaultCacheWriterConfig.getWriteBatchSize(), configDefault.getCacheWriterConfiguration().getWriteBatchSize());
         assertEquals(defaultCacheWriterConfig.getRetryAttempts(), configDefault.getCacheWriterConfiguration().getRetryAttempts());
-        assertEquals(defaultCacheWriterConfig.getRetryAttemptDelaySeconds(), configDefault.getCacheWriterConfiguration().getRetryAttemptDelaySeconds());
+        assertEquals(defaultCacheWriterConfig.getRetryAttemptDelaySeconds(), configDefault.getCacheWriterConfiguration()
+                .getRetryAttemptDelaySeconds());
 
         Ehcache defaultCache = configurationHelper.createDefaultCache();
         assertEquals("default", defaultCache.getName());
@@ -1508,8 +1554,10 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         CacheConfiguration config1 = configs.get("writeThroughCache1");
         assertNotNull(config1.getCacheWriterConfiguration());
         assertEquals(defaultCacheWriterConfig.getWriteMode(), config1.getCacheWriterConfiguration().getWriteMode());
-        assertEquals(defaultCacheWriterConfig.getCacheWriterFactoryConfiguration(), config1.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration());
-        assertEquals(defaultCacheWriterConfig.getNotifyListenersOnException(), config1.getCacheWriterConfiguration().getNotifyListenersOnException());
+        assertEquals(defaultCacheWriterConfig.getCacheWriterFactoryConfiguration(), config1.getCacheWriterConfiguration()
+                .getCacheWriterFactoryConfiguration());
+        assertEquals(defaultCacheWriterConfig.getNotifyListenersOnException(), config1.getCacheWriterConfiguration()
+                .getNotifyListenersOnException());
         assertEquals(defaultCacheWriterConfig.getMaxWriteDelay(), config1.getCacheWriterConfiguration().getMaxWriteDelay());
         assertEquals(defaultCacheWriterConfig.getRateLimitPerSecond(), config1.getCacheWriterConfiguration().getRateLimitPerSecond());
         assertEquals(defaultCacheWriterConfig.getWriteCoalescing(), config1.getCacheWriterConfiguration().getWriteCoalescing());
@@ -1521,8 +1569,10 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         CacheConfiguration config2 = configs.get("writeThroughCache2");
         assertNotNull(config2.getCacheWriterConfiguration());
         assertEquals(defaultCacheWriterConfig.getWriteMode(), config2.getCacheWriterConfiguration().getWriteMode());
-        assertEquals(defaultCacheWriterConfig.getCacheWriterFactoryConfiguration(), config2.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration());
-        assertEquals(defaultCacheWriterConfig.getNotifyListenersOnException(), config2.getCacheWriterConfiguration().getNotifyListenersOnException());
+        assertEquals(defaultCacheWriterConfig.getCacheWriterFactoryConfiguration(), config2.getCacheWriterConfiguration()
+                .getCacheWriterFactoryConfiguration());
+        assertEquals(defaultCacheWriterConfig.getNotifyListenersOnException(), config2.getCacheWriterConfiguration()
+                .getNotifyListenersOnException());
         assertEquals(defaultCacheWriterConfig.getMaxWriteDelay(), config2.getCacheWriterConfiguration().getMaxWriteDelay());
         assertEquals(defaultCacheWriterConfig.getRateLimitPerSecond(), config2.getCacheWriterConfiguration().getRateLimitPerSecond());
         assertEquals(defaultCacheWriterConfig.getWriteCoalescing(), config2.getCacheWriterConfiguration().getWriteCoalescing());
@@ -1534,7 +1584,8 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         CacheConfiguration config3 = configs.get("writeThroughCache3");
         assertNotNull(config3.getCacheWriterConfiguration());
         assertEquals(CacheWriterConfiguration.WriteMode.WRITE_THROUGH, config3.getCacheWriterConfiguration().getWriteMode());
-        assertEquals(defaultCacheWriterConfig.getCacheWriterFactoryConfiguration(), config3.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration());
+        assertEquals(defaultCacheWriterConfig.getCacheWriterFactoryConfiguration(), config3.getCacheWriterConfiguration()
+                .getCacheWriterFactoryConfiguration());
         assertEquals(true, config3.getCacheWriterConfiguration().getNotifyListenersOnException());
         assertEquals(30, config3.getCacheWriterConfiguration().getMaxWriteDelay());
         assertEquals(10, config3.getCacheWriterConfiguration().getRateLimitPerSecond());
@@ -1547,7 +1598,8 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         CacheConfiguration config4 = configs.get("writeThroughCache4");
         assertNotNull(config4.getCacheWriterConfiguration());
         assertEquals(CacheWriterConfiguration.WriteMode.WRITE_THROUGH, config4.getCacheWriterConfiguration().getWriteMode());
-        assertEquals("net.sf.ehcache.writer.TestCacheWriterFactory", config4.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration().getFullyQualifiedClassPath());
+        assertEquals("net.sf.ehcache.writer.TestCacheWriterFactory", config4.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration()
+                .getFullyQualifiedClassPath());
         assertEquals(null, config4.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration().getProperties());
         assertEquals(null, config4.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration().getPropertySeparator());
         assertEquals(false, config4.getCacheWriterConfiguration().getNotifyListenersOnException());
@@ -1562,8 +1614,10 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         CacheConfiguration config5 = configs.get("writeBehindCache5");
         assertNotNull(config5.getCacheWriterConfiguration());
         assertEquals(CacheWriterConfiguration.WriteMode.WRITE_BEHIND, config5.getCacheWriterConfiguration().getWriteMode());
-        assertEquals("net.sf.ehcache.writer.TestCacheWriterFactory", config5.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration().getFullyQualifiedClassPath());
-        assertEquals("just.some.property=test; another.property=test2", config5.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration().getProperties());
+        assertEquals("net.sf.ehcache.writer.TestCacheWriterFactory", config5.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration()
+                .getFullyQualifiedClassPath());
+        assertEquals("just.some.property=test; another.property=test2", config5.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration()
+                .getProperties());
         assertEquals(";", config5.getCacheWriterConfiguration().getCacheWriterFactoryConfiguration().getPropertySeparator());
         assertEquals(true, config5.getCacheWriterConfiguration().getNotifyListenersOnException());
         assertEquals(8, config5.getCacheWriterConfiguration().getMaxWriteDelay());
@@ -1574,7 +1628,7 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         assertEquals(2, config5.getCacheWriterConfiguration().getRetryAttempts());
         assertEquals(2, config5.getCacheWriterConfiguration().getRetryAttemptDelaySeconds());
         Ehcache cache5 = configurationHelper.createCacheFromName("writeBehindCache5");
-        Properties properties5 = ((TestCacheWriter)cache5.getRegisteredCacheWriter()).getProperties();
+        Properties properties5 = ((TestCacheWriter) cache5.getRegisteredCacheWriter()).getProperties();
         assertEquals(2, properties5.size());
         assertEquals("test", properties5.getProperty("just.some.property"));
         assertEquals("test2", properties5.getProperty("another.property"));
@@ -1600,5 +1654,36 @@ public class ConfigurationFactoryTest extends AbstractCacheTest {
         helpTestListenFor(configuration, "listenLocal", NotificationScope.LOCAL);
         helpTestListenFor(configuration, "listenRemote", NotificationScope.REMOTE);
     }
-    
+
+
+    @Test
+    public void testCacheConfigurationWithNoName() {
+
+        //Don't set cache name
+        CacheConfiguration cacheConfigurationTest3Cache = new CacheConfiguration().maxElementsInMemory(10)
+                .eternal(true).memoryStoreEvictionPolicy(MemoryStoreEvictionPolicy.LRU).overflowToDisk(false)
+                .statistics(false);
+
+        try {
+            Cache cache = new Cache(cacheConfigurationTest3Cache);
+        } catch (InvalidConfigurationException e) {
+            //expected
+        }
+
+    }
+
+    @Test
+    public void testValidStoreConfigElements() throws Exception {
+        File file = new File(TEST_CONFIG_DIR + "ehcache-store.xml");
+        Configuration configuration = ConfigurationFactory.parseConfiguration(file);
+
+        CacheConfiguration cacheConfiguration = configuration.getCacheConfigurations().get("offheap1");
+        assertEquals(16777216, cacheConfiguration.getMaxMemoryOffHeapInBytes());
+        assertEquals(true, cacheConfiguration.isOverflowToOffHeap());
+
+        cacheConfiguration = configuration.getCacheConfigurations().get("offheap2");
+        assertEquals(2147483648L, cacheConfiguration.getMaxMemoryOffHeapInBytes());
+        assertEquals(false, cacheConfiguration.isOverflowToOffHeap());
+    }
+
 }
