@@ -21,18 +21,27 @@ import net.sf.ehcache.CacheException;
 import net.sf.ehcache.CacheManager;
 import net.sf.ehcache.Ehcache;
 import net.sf.ehcache.Element;
+import net.sf.ehcache.util.RetryAssert;
+import org.hamcrest.CoreMatchers;
+import org.hamcrest.core.Is;
 import org.hibernate.cfg.Environment;
 import org.junit.After;
+
+import static java.util.concurrent.TimeUnit.SECONDS;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertThat;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
+
 import org.junit.Test;
 
 import java.io.IOException;
 import java.io.Serializable;
 import java.util.Map;
 import java.util.Properties;
+import java.util.concurrent.Callable;
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -41,12 +50,10 @@ import org.slf4j.LoggerFactory;
  * Tests for a Cache
  *
  * @author Greg Luck, Claus Ibsen
- * @version $Id: HibernateAPIUsageTest.java 2154 2010-04-06 02:45:52Z cdennis $
+ * @version $Id: HibernateAPIUsageTest.java 4670 2011-09-09 22:46:40Z alexsnaps $
  */
 public class HibernateAPIUsageTest extends AbstractCacheTest {
     private static final Logger LOG = LoggerFactory.getLogger(HibernateAPIUsageTest.class.getName());
-    private static final int EMPTY_ELEMENT_SIZE = 228;
-
 
     /**
      * teardown
@@ -86,7 +93,7 @@ public class HibernateAPIUsageTest extends AbstractCacheTest {
      * getTimeout appears to be broken. It returns 4096 minutes!
      */
     @Test
-    public void testAPIAsUsedByHibernate3() {
+    public void testAPIAsUsedByHibernate3() throws InterruptedException {
 
         /*Shutdown cache manager so that hibernate can start one using the same ehcache.xml disk path
           because it does not use the singleton CacheManager any more */
@@ -94,15 +101,18 @@ public class HibernateAPIUsageTest extends AbstractCacheTest {
 
         org.hibernate.cache.EhCacheProvider provider = new org.hibernate.cache.EhCacheProvider();
         provider.start(null);
-        org.hibernate.cache.Cache cache = provider.buildCache("sampleCache1", null);
+        final org.hibernate.cache.Cache cache = provider.buildCache("sampleCache1", null);
+        final Serializable key = "key";
+        final Serializable value = "value";
+        assertThat(cache.getSizeInMemory(), CoreMatchers.is(0L));
+        cache.put(key, value);
+        final long EMPTY_ELEMENT_SIZE = cache.getSizeInMemory();
+        cache.clear();
 
         //Check created and name
         assertNotNull(cache.getRegionName());
         assertEquals("sampleCache1", cache.getRegionName());
 
-        Serializable key = "key";
-        Serializable value = "value";
-
         cache.put(key, value);
         assertEquals(value, cache.get(key));
         assertEquals(value, cache.read(key));
@@ -119,9 +129,14 @@ public class HibernateAPIUsageTest extends AbstractCacheTest {
         for (int i = 0; i < 10010; i++) {
             cache.put("" + i, value);
         }
+        Thread.sleep(100);
         //this is now fixed
         assertEquals(10000, cache.getElementCountInMemory());
-        assertEquals(10, cache.getElementCountOnDisk());
+        RetryAssert.assertBy(1, SECONDS, new Callable<Long>() {
+            public Long call() throws Exception {
+                return cache.getElementCountOnDisk();
+            }
+        }, Is.is(1000L));
 
         //clear
         cache.clear();
@@ -228,8 +243,6 @@ public class HibernateAPIUsageTest extends AbstractCacheTest {
         cache.clear();
         assertEquals(0, cache.getElementCountInMemory());
         cache.put(key, value);
-        //Not Serializable therefore unmeasurable using ehcache's estimation algorithm
-        assertTrue(0 == cache.getSizeInMemory());
 
         //locks
         //timeout. This seems strange
@@ -285,15 +298,18 @@ public class HibernateAPIUsageTest extends AbstractCacheTest {
 
         properties.setProperty("net.sf.ehcache.configurationResourceName", "ehcache-2.xml");
         provider.start(properties);
-        org.hibernate.cache.Cache cache = provider.buildCache("sampleCache1", null);
+        final org.hibernate.cache.Cache cache = provider.buildCache("sampleCache1", null);
+        final Serializable key = "key";
+        final Serializable value = "value";
+        assertThat(cache.getSizeInMemory(), CoreMatchers.is(0L));
+        cache.put(key, value);
+        final long EMPTY_ELEMENT_SIZE = cache.getSizeInMemory();
+        cache.clear();
 
         //Check created and name
         assertNotNull(cache.getRegionName());
         assertEquals("sampleCache1", cache.getRegionName());
 
-        Serializable key = "key";
-        Serializable value = "value";
-
         cache.put(key, value);
         assertEquals(value, cache.get(key));
         assertEquals(value, cache.read(key));
@@ -311,7 +327,11 @@ public class HibernateAPIUsageTest extends AbstractCacheTest {
             cache.put("" + i, value);
         }
         assertEquals(10000, cache.getElementCountInMemory());
-        assertEquals(10, cache.getElementCountOnDisk());
+        RetryAssert.assertBy(2, SECONDS, new Callable<Long>() {
+                public Long call() throws Exception {
+                    return cache.getElementCountOnDisk();
+                }
+            }, Is.is(10010L));
 
         //clear
         cache.clear();
@@ -483,8 +503,8 @@ public class HibernateAPIUsageTest extends AbstractCacheTest {
         cache.clear();
         assertEquals(0, cache.getElementCountInMemory());
         cache.put(key, value);
-        //Not Serializable therefore unmeasurable using ehcache's estimation algorithm
-        assertTrue(0 == cache.getSizeInMemory());
+        // elements don't need to be serializable anymore to be measured
+        assertTrue(cache.getSizeInMemory() > 0);
 
         //locks
         //timeout. This seems strange
@@ -584,8 +604,8 @@ public class HibernateAPIUsageTest extends AbstractCacheTest {
         cache.clear();
         assertEquals(0, cache.getElementCountInMemory());
         cache.put(key, value);
-        //Not Serializable therefore unmeasurable using ehcache's estimation algorithm
-        assertTrue(0 == cache.getSizeInMemory());
+        // elements don't need to be serializable anymore to be measured
+        assertTrue(cache.getSizeInMemory() > 0);
 
         //locks
         //timeout. This seems strange
